@@ -30,28 +30,30 @@ from __future__ import unicode_literals
 import csv, logging
 
 from django.contrib import messages
-from rest_framework.generics import (CreateAPIView,
-    GenericAPIView, RetrieveAPIView)
+from rest_framework import generics
 from rest_framework.mixins import CreateModelMixin
-from rest_framework.response import Response
+from rest_framework import response as http
 from rest_framework import status
 
 from ..backends import ProcessorError
 from ..compat import gettext_lazy as _, is_authenticated, StringIO
-from ..docs import swagger_auto_schema, OpenAPIResponse
-from ..mixins import BalanceAndCartMixin, CartMixin, InvoicablesMixin
+from ..docs import extend_schema, OpenApiResponse
+from ..filters import DateRangeFilter, OrderingFilter, SearchFilter
+from ..mixins import (BalanceAndCartMixin, CartMixin, InvoicablesMixin,
+                      UserMixin)
 from ..models import CartItem, get_broker
-from ..utils import datetime_or_now
+from ..utils import datetime_or_now, get_user_serializer
 from .serializers import (CartItemSerializer, CartItemCreateSerializer,
     CartItemUploadSerializer, ChargeSerializer, CheckoutSerializer,
     OrganizationCartSerializer, RedeemCouponSerializer,
-    ValidationErrorSerializer)
+    ValidationErrorSerializer, CartItemUpdateSerializer,
+    UserCartItemCreateSerializer, QueryParamCartItemSerializer)
 
 
 LOGGER = logging.getLogger(__name__)
 
 
-class CartItemAPIView(CartMixin, CreateAPIView):
+class CartItemAPIView(CartMixin, generics.CreateAPIView):
     """
     Adds an item to the user cart
 
@@ -119,21 +121,19 @@ class CartItemAPIView(CartMixin, CreateAPIView):
     # and csrf, unfortunately it prevents authenticated users to add into
     # their db cart, instead put their choices into the unauth session.
     # authentication_classes = []
-    @swagger_auto_schema(responses={
-      200: OpenAPIResponse("updated", CartItemSerializer),
-      201: OpenAPIResponse("created", CartItemSerializer)})
+    @extend_schema(responses={
+      200: OpenApiResponse(CartItemSerializer),
+      201: OpenApiResponse(CartItemSerializer)})
     def post(self, request, *args, **kwargs):
         items = None
-        serializer = self.get_serializer(data=request.data)
-        if serializer.is_valid():
-            items = [serializer.validated_data]
+        if isinstance(request.data, dict):
+            serializer = self.get_serializer(data=request.data)
+            if serializer.is_valid(raise_exception=True):
+                items = [serializer.validated_data]
         else:
-            serializer = self.get_serializer(data=request.data, many=True)
-            if serializer.is_valid():
-                items = serializer.validated_data
-        if not items:
-            return Response(
-                serializer.errors, status=status.HTTP_400_BAD_REQUEST)
+            serializer_list = self.get_serializer(data=request.data, many=True)
+            if serializer_list.is_valid(raise_exception=True):
+                items = serializer_list.validated_data
 
         cart_items = []
         at_time = datetime_or_now()
@@ -151,9 +151,9 @@ class CartItemAPIView(CartMixin, CreateAPIView):
             cart_items += [cart_item]
         if len(items) > 1:
             headers = self.get_success_headers(cart_items)
-            return Response(cart_items, status=status_code, headers=headers)
+            return http.Response(cart_items, status=status_code, headers=headers)
         headers = self.get_success_headers(cart_items[0])
-        return Response(cart_items[0], status=status_code, headers=headers)
+        return http.Response(cart_items[0], status=status_code, headers=headers)
 
     @staticmethod
     def destroy_in_session(request, plan=None, email=None):
@@ -181,6 +181,7 @@ class CartItemAPIView(CartMixin, CreateAPIView):
             kwargs.update({'email': email})
         CartItem.objects.get_cart(request.user, **kwargs).delete()
 
+    @extend_schema(parameters=[QueryParamCartItemSerializer])
     def delete(self, request, *args, **kwargs):
         """
         Removes an item from the user cart
@@ -196,19 +197,21 @@ class CartItemAPIView(CartMixin, CreateAPIView):
             DELETE /api/cart?plan=premium HTTP/1.1
         """
         #pylint:disable=unused-argument
-        plan = None
-        email = None
-        plan = request.query_params.get('plan')
-        email = request.query_params.get('email')
+        query_serializer = QueryParamCartItemSerializer(
+            data=request.query_params)
+        query_serializer.is_valid(raise_exception=True)
+        plan = query_serializer.validated_data.get('plan', None)
+        email = query_serializer.validated_data.get('email', None)
+
         self.destroy_in_session(request, plan=plan, email=email)
         if is_authenticated(request):
             # If the user is authenticated, we delete the cart items
             # from the database.
             self.destroy_in_db(request, plan=plan, email=email)
-        return Response(status=status.HTTP_204_NO_CONTENT)
+        return http.Response(status=status.HTTP_204_NO_CONTENT)
 
 
-class CartItemUploadAPIView(CartMixin, GenericAPIView):
+class CartItemUploadAPIView(CartMixin, generics.GenericAPIView):
     """
     Uploads multiple items into a user cart
 
@@ -321,10 +324,10 @@ class CartItemUploadAPIView(CartMixin, GenericAPIView):
                     response['failed'].append({'data': serializer.data,
                                                'error': serializer.errors})
 
-        return Response(response, status=status_code)
+        return http.Response(response, status=status_code)
 
 
-class CouponRedeemAPIView(GenericAPIView):
+class CouponRedeemAPIView(generics.GenericAPIView):
     """
     Redeems a discount code
 
@@ -358,8 +361,8 @@ class CouponRedeemAPIView(GenericAPIView):
     # XXX This is not a ValidationErrorSerializer but we return a message.
     # XXX Should many return the updated cart but we are dealing with users,
     # not organizations here.
-    @swagger_auto_schema(responses={
-        200: OpenAPIResponse("", ValidationErrorSerializer)})
+    @extend_schema(responses={
+        200: OpenApiResponse(ValidationErrorSerializer)})
     def post(self, request, *args, **kwargs): #pylint: disable=unused-argument
         serializer = self.get_serializer(data=request.data)
         if serializer.is_valid():
@@ -376,17 +379,17 @@ class CouponRedeemAPIView(GenericAPIView):
                 # of this statement just yet.
                 #pylint: disable=protected-access
                 messages.success(request._request, details['detail'])
-                return Response(details, status=status.HTTP_200_OK,
+                return http.Response(details, status=status.HTTP_200_OK,
                                 headers=headers)
             details = {'detail': (
                 _("No items can be discounted using this coupon: %(code)s.") % {
                 'code': coupon_code})}
-            return Response(details, status=status.HTTP_400_BAD_REQUEST)
-        return Response(serializer.errors, status=status.HTTP_400_BAD_REQUEST)
+            return http.Response(details, status=status.HTTP_400_BAD_REQUEST)
+        return http.Response(serializer.errors, status=status.HTTP_400_BAD_REQUEST)
 
 
 class CheckoutAPIView(InvoicablesMixin, BalanceAndCartMixin,
-                      CreateModelMixin, RetrieveAPIView):
+                      CreateModelMixin, generics.RetrieveAPIView):
     """
     Retrieves a cart for checkout
 
@@ -470,8 +473,8 @@ class CheckoutAPIView(InvoicablesMixin, BalanceAndCartMixin,
             return CheckoutSerializer
         return super(CheckoutAPIView, self).get_serializer_class()
 
-    @swagger_auto_schema(responses={
-        201: OpenAPIResponse("", ChargeSerializer)})
+    @extend_schema(responses={
+        201: OpenApiResponse(ChargeSerializer)})
     def post(self, request, *args, **kwargs):
         """
         Checkouts a cart
@@ -539,7 +542,7 @@ of Xia",
             'results': self.get_queryset()
         }
         serializer = self.get_serializer(resp_data)
-        return Response(serializer.data)
+        return http.Response(serializer.data)
 
     def create(self, request, *args, **kwargs):#pylint:disable=unused-argument
         serializer = CheckoutSerializer(data=request.data)
@@ -569,8 +572,393 @@ of Xia",
                 remember_card=data.get('remember_card', False))
             if charge and charge.invoiced_total.amount > 0:
                 result = ChargeSerializer(charge)
-                return Response(result.data, status=status.HTTP_200_OK)
+                return http.Response(result.data, status=status.HTTP_200_OK)
         except ProcessorError as err:
-            return Response({
+            return http.Response({
                 'detail': str(err)}, status=status.HTTP_400_BAD_REQUEST)
-        return Response({}, status=status.HTTP_200_OK)
+        return http.Response({}, status=status.HTTP_200_OK)
+
+
+class ActiveCartItemListCreateView(generics.ListCreateAPIView):
+    """
+    Lists active cart items
+
+    Returns a list of {{PAGE_SIZE}} cart items that haven't been checked out
+    yet.
+
+    The queryset can be further refined to match a search filter (``q``)
+    and/or a range of dates ([``start_at``, ``ends_at``]),
+    and sorted on specific fields (``o``).
+
+    **Tags**: billing, broker, cart
+
+    **Examples**
+
+    .. code-block:: http
+
+        GET /api/billing/cartitems HTTP/1.1
+
+    responds
+
+    .. code-block:: json
+
+        {
+          "count": 1,
+          "next": null,
+          "previous": null,
+          "results": [{
+            "created_at": "2023-10-11T21:20:06.444545-05:00",
+            "user": {
+              "username": "xia",
+              "slug": "xia",
+              "email": "xia@example.com",
+              "full_name": "Xia Lee",
+              "created_at": "2023-09-06T21:49:28.003319-05:00",
+              "last_login": "2023-10-11T03:31:10.138177-05:00"
+            },
+            "plan": {
+              "slug": "basic",
+              "title": "Basic"
+            },
+            "option": 0,
+            "use": null,
+            "quantity": 1,
+            "sync_on": null,
+            "full_name": "Xia Lee",
+            "email": "xia@example.com",
+            "detail": null
+          }]
+        }
+    """
+    search_fields = (
+        'user__username',
+        'user__first_name',
+        'user__last_name',
+        'user__email',
+    )
+    ordering_fields = (
+        ('user__username', 'username'),
+        ('created_at', 'created_at')
+    )
+    ordering = ('created_at',)
+
+    filter_backends = (DateRangeFilter, SearchFilter, OrderingFilter)
+    queryset = CartItem.objects.filter(recorded=False).select_related(
+        'user', 'plan')
+    serializer_class = CartItemSerializer
+
+    def get_serializer_class(self):
+        if self.request.method.lower() in ['post']:
+            return UserCartItemCreateSerializer
+        return CartItemSerializer
+
+
+    @extend_schema(responses={
+        201: OpenApiResponse(CartItemSerializer)})
+    def post(self, request, *args, **kwargs):
+        """
+        Creates a cart item
+
+        This endpoint lets broker to add items into a user cart.
+
+        **Tags**: billing, broker, cart
+
+        **Examples**
+
+        .. code-block:: http
+
+            POST /api/billing/cartitems HTTP/1.1
+
+        .. code-block:: json
+
+            {
+                "user": "xia",
+                "plan": "basic"
+            }
+
+        responds
+
+        .. code-block:: json
+
+            {
+              "user": {
+                "username": "xia",
+                "slug": "xia",
+                "email": "xia@example.com",
+                "full_name": "Xia Lee",
+                "created_at": "2023-09-06T21:49:28.003319-05:00",
+                "last_login": "2023-10-11T03:31:10.138177-05:00"
+              },
+              "plan": {
+                "slug": "basic",
+                "title": "Basic"
+              },
+              "created_at": "2023-10-12T05:47:17.421103-05:00",
+              "option": 3,
+              "use": null,
+              "quantity": 50,
+              "sync_on": "",
+              "full_name": "Xia Lee",
+              "email": "xia@example.com",
+              "detail": "Cart item created"
+            }
+        """
+        response = super(ActiveCartItemListCreateView, self).create(
+            request, *args, **kwargs)
+        if response.status_code == status.HTTP_201_CREATED:
+            response.data['detail'] = _('Cart item created')
+
+        return http.Response(response.data, status=status.HTTP_201_CREATED,
+                             headers=response.headers)
+
+
+class ActiveCartItemRetrieveUpdateDestroyView(
+        generics.RetrieveUpdateDestroyAPIView):
+    """
+    Retrieves a cart item
+
+    Returns a single cart item based on its primary key.
+
+    This API endpoint is intended for broker to analyze and modify
+    active cart items as necessary when responding to support requests.
+
+    **Tags**: billing, broker, cart
+
+    **Examples**
+
+    .. code-block:: http
+
+        GET /api/billing/cartitems/{cartitem_id} HTTP/1.1
+
+    responds
+
+    .. code-block:: json
+
+        {
+          "user": {
+            "username": "xia",
+            "slug": "xia",
+            "email": "xia@example.com",
+            "full_name": "Xia Lee",
+            "created_at": "2023-09-06T21:49:28.003319-05:00",
+            "last_login": "2023-10-11T03:31:10.138177-05:00"
+          },
+          "plan": {
+            "slug": "basic",
+            "title": "Basic"
+          },
+          "created_at": "2023-10-11T23:22:54.407880-05:00",
+          "option": 3,
+          "use": null,
+          "quantity": 50,
+          "sync_on": null,
+          "full_name": "Xia Lee",
+          "email": "xia@example.com"
+        }
+    """
+    lookup_field = "id"
+    lookup_url_kwarg = "cartitem_id"
+    queryset = CartItem.objects.filter(recorded=False).select_related(
+        'user', 'plan')
+
+    def get_serializer_class(self):
+        if self.request.method.lower() in ['put', 'patch']:
+            return CartItemUpdateSerializer
+        return CartItemSerializer
+
+
+    def delete(self, request, *args, **kwargs):
+        """
+        Deletes a cart item
+
+        Deletes a single cart item based on its primary key.
+
+        This API endpoint is intended for broker to analyze and modify
+        active cart items as necessary when responding to support requests.
+
+        **Tags**: billing, broker, cart
+
+        **Examples**
+
+        .. code-block:: http
+
+            DELETE /api/billing/cartitems/{cartitem_id} HTTP/1.1
+        """
+        return self.destroy(request, *args, **kwargs)
+
+
+    @extend_schema(responses={
+        200: OpenApiResponse(CartItemUpdateSerializer)})
+    def put(self, request, *args, **kwargs):
+        """
+        Updates a cart item
+
+        Updates a single cart item based on its primary key.
+
+        This API endpoint is intended for broker to analyze and modify
+        active cart items as necessary when responding to support requests.
+
+        **Tags**: billing, broker, cart
+
+        **Examples**
+
+        .. code-block:: http
+
+            PUT /api/billing/cartitems/{cartitem_id} HTTP/1.1
+
+        .. code-block:: json
+
+            {
+              "option": 2,
+              "quantity": 25
+            }
+
+        responds
+
+        .. code-block:: json
+
+            {
+                "created_at": "2023-10-12T02:04:49.151044-05:00",
+                "user": {
+                    "slug": "xia",
+                    "email": "xia@example.com",
+                    "full_name": "Xia Lee",
+                    "created_at": "2023-09-06T21:58:07.969908-05:00",
+                    "last_login": "2023-10-12T01:49:13.092566-05:00"
+                },
+                "plan": {
+                    "slug": "basic",
+                    "title": "Basic"
+                },
+                "option": 2,
+                "use": null,
+                "quantity": 25,
+                "sync_on": "",
+                "full_name": "",
+                "email": "",
+                "detail": "Cart item updated"
+            }
+        """
+        response = self.update(request, *args, **kwargs)
+        if response.status_code == status.HTTP_200_OK:
+            response.data['detail'] = _('Cart item updated')
+
+        return http.Response(response.data, status=response.status_code)
+
+
+class UserCartItemListView(UserMixin, generics.ListAPIView):
+    """
+    Lists a user active cart items
+
+    Returns a list of {{PAGE_SIZE}} cart items that a specified user hasn't
+    checked out yet.
+
+    The queryset can be further refined to match a search filter (``q``)
+    and/or a range of dates ([``start_at``, ``ends_at``]),
+    and sorted on specific fields (``o``).
+
+    **Tags**: billing, broker, cart
+
+    **Examples**
+
+    .. code-block:: http
+
+        GET /api/billing/cartitems/user/xia HTTP/1.1
+
+    responds
+
+    .. code-block:: json
+
+        {
+            "count": 1,
+            "next": null,
+            "previous": null,
+            "results": [
+                {
+                    "created_at": "2023-10-11T23:49:59.485511-05:00",
+                    "user": {
+                        "username": "xia",
+                        "slug": "xia",
+                        "email": "xia@example.com",
+                        "full_name": "Xia Lee",
+                        "created_at": "2023-09-06T21:49:28.003319-05:00",
+                        "last_login": "2023-10-11T03:31:10.138177-05:00"
+                    },
+                    "plan": {
+                        "slug": "basic",
+                        "title": "Basic"
+                    },
+                    "option": 0,
+                    "use": null,
+                    "quantity": 2,
+                    "sync_on": null,
+                    "full_name": "",
+                    "email": null,
+                    "id": 137
+                }]
+            }
+        """
+    search_fields = (
+        'user__username',
+        'user__first_name',
+        'user__last_name',
+        'user__email',
+    )
+    ordering_fields = (
+        ('user__username', 'username'),
+        ('created_at', 'created_at')
+    )
+    ordering = ('created_at',)
+
+    filter_backends = (DateRangeFilter, SearchFilter, OrderingFilter)
+    queryset = CartItem.objects.filter(recorded=False).select_related(
+        'user', 'plan')
+    serializer_class = CartItemSerializer
+
+    def get_serializer(self, *args, **kwargs):
+        serializer = super(UserCartItemListView, self).get_serializer(
+            *args, **kwargs)
+        # Check if the serializer has a 'child' attribute (indicating it's a
+        # ListSerializer)
+        if hasattr(serializer, 'child'):
+            # Accessing child serializer because ListSerializer is the
+            # parent serializer for querysets.
+            # https://www.django-rest-framework.org/api-guide/serializers/#listserializer
+            # The child serializer is then responsible for each individual
+            # object. Another way to do it would be to loop through each item
+            # in the queryset and remove "user" from the representation,
+            # but this might be more efficient.
+            child_serializer = serializer.child
+            original_to_representation = child_serializer.to_representation
+
+            def _to_representation(instance):
+                # Adding the "id" field
+                representation = original_to_representation(instance)
+                representation['id'] = instance.id
+                # Removing the "user" field
+                representation.pop('user', None)
+                return representation
+
+            child_serializer.to_representation = _to_representation
+        return serializer
+
+    def get_queryset(self):
+        user = self.user if self.user is not None else self.request.user
+        # The queryset = (...) returns a queryset meaning
+        # ListSerializer is automatically used
+        queryset = self.queryset.filter(user=user)
+        return queryset
+
+    def list(self, request, *args, **kwargs):
+        response = super(UserCartItemListView, self).list(
+            request, *args, **kwargs)
+        user = self.user if self.user is not None else self.request.user
+        user_data = get_user_serializer()(user).data
+        # Adding user_data here otherwise it gets repeated for each cartitem
+        # Adding all items except the "results" in order to show the
+        # User field before the "results."
+        data = {k: response.data[k] for k in list(response.data.keys())[:-1]}
+        data['user'] = user_data
+        data['results'] = response.data['results']
+
+        return http.Response(data)
