@@ -28,31 +28,236 @@ from rest_framework.generics import GenericAPIView, ListAPIView
 from rest_framework.response import Response
 
 from .serializers import (CartItemSerializer, LifetimeSerializer,
-    MetricsSerializer)
-from .. import settings
+    MetricsSerializer, QueryParamPeriodSerializer, BalancesDueSerializer)
+from .. import settings, humanize
 from ..compat import gettext_lazy as _, reverse, six
+from ..docs import extend_schema
 from ..filters import DateRangeFilter
-from ..metrics.base import (abs_monthly_balances,
-    aggregate_transactions_by_period, month_periods,
-    aggregate_transactions_change_by_period, get_different_units)
-from ..metrics.subscriptions import (active_subscribers, churn_subscribers,
-    subscribers_age)
+from ..metrics.base import (abs_balances_by_period,
+    aggregate_transactions_by_period, aggregate_transactions_change_by_period,
+    generate_periods, get_different_units)
+from ..metrics.subscriptions import (active_subscribers_by_period,
+    churn_subscribers_by_period, subscribers_age)
 from ..metrics.transactions import lifetime_value
 from ..mixins import (CartItemSmartListMixin, CouponMixin,
-    ProviderMixin, DateRangeContextMixin)
+    ProviderMixin, DateRangeContextMixin, BalancesDueMixin)
 from ..models import CartItem, Plan, Transaction
 from ..utils import convert_dates_to_utc, get_organization_model
 
 LOGGER = logging.getLogger(__name__)
 
 
-class BalancesAPIView(DateRangeContextMixin, ProviderMixin,
-                      GenericAPIView):
+class MetricsMixin(DateRangeContextMixin, ProviderMixin):
     """
-    Retrieves 12-month trailing deferred balances
+    Base class for metrics APIs
+    """
+    period_type_param = 'period_type'
+    nb_periods_param = 'nb_periods'
+    serializer_class = MetricsSerializer
 
-    Generate a table of revenue (rows) per months (columns) for a default
-    balance sheet (Income, Backlog, Receivable).
+    def calculate_date_periods(self):
+        # We use self.get_query_param since this Mixin is also used
+        # by the CSVDownloadViews, which sends in a Django request instead
+        # of a DRF request
+        nb_periods = self.get_query_param(self.nb_periods_param)
+        period_type = self.get_query_param(self.period_type_param)
+
+        query_params = {}
+        if nb_periods:
+            query_params[self.nb_periods_param] = nb_periods
+        if period_type:
+            query_params[self.period_type_param] = period_type
+
+        query_serializer = QueryParamPeriodSerializer(data=query_params)
+        query_serializer.is_valid(raise_exception=True)
+
+        nb_periods = query_serializer.validated_data.get(
+            self.nb_periods_param)
+        period_type = query_serializer.validated_data.get(
+            self.period_type_param, humanize.MONTHLY)
+
+        date_periods = convert_dates_to_utc(
+            generate_periods(period_type, nb_periods=nb_periods,
+                             from_date=self.ends_at,
+                             tzinfo=self.organization.default_timezone))
+
+        return date_periods
+
+
+class BalancesMetricsMixin(MetricsMixin):
+
+    def get_data(self):
+        results = []
+
+        date_periods = self.calculate_date_periods()
+        unit = settings.DEFAULT_UNIT
+        for key in [Transaction.INCOME, Transaction.BACKLOG,
+                    Transaction.RECEIVABLE]:
+            values, _unit = abs_balances_by_period(
+                organization=self.provider, account=key,
+                date_periods=date_periods)
+
+            if _unit:
+                unit = _unit
+
+            results += [{
+                'slug': key,
+                'title': key,
+                'values': values
+            }]
+
+        return results, unit
+
+    def retrieve_metrics(self):
+        results, unit = self.get_data()
+
+        return Response({
+            'title': "Balances",
+            'unit': unit,
+            'scale': 0.01,
+            'results': results
+        })
+
+
+class RevenueMetricsMixin(MetricsMixin):
+
+    def get_data(self):
+        unit = settings.DEFAULT_UNIT
+
+        date_periods = self.calculate_date_periods()
+
+        account_table, _, _, table_unit = \
+            aggregate_transactions_change_by_period(self.provider,
+                Transaction.RECEIVABLE, account_title='Sales',
+                orig='orig', dest='dest',
+                date_periods=date_periods)
+
+        _, payment_amounts, payments_unit = aggregate_transactions_by_period(
+            self.provider, Transaction.RECEIVABLE,
+            orig='dest', dest='dest',
+            orig_account=Transaction.BACKLOG,
+            orig_organization=self.provider,
+            date_periods=date_periods)
+
+        _, refund_amounts, refund_unit = aggregate_transactions_by_period(
+            self.provider, Transaction.REFUND,
+            orig='dest', dest='dest',
+            date_periods=date_periods)
+
+        units = get_different_units(table_unit, payments_unit, refund_unit)
+
+        if len(units) > 1:
+            LOGGER.error("different units in RevenueMetricAPIView.get: %s",
+                units)
+
+        if units:
+            unit = units[0]
+
+        account_table += [{
+            'slug': "payments",
+            'title': "Payments",
+            'values': payment_amounts
+        }, {
+            'slug': "refunds",
+            'title': "Refunds",
+            'values': refund_amounts
+        }]
+
+        return account_table, unit
+
+    def retrieve_metrics(self):
+        account_table, unit = self.get_data()
+        resp = {
+            'title': "Amount",
+            'unit': unit,
+            'scale': 0.01,
+            'results': account_table
+        }
+        if not self.provider.has_bank_account:
+            resp.update({'processor_hint': 'connect_provider'})
+
+        return Response(resp)
+
+
+class CustomerMetricsMixin(MetricsMixin):
+
+    def get_data(self):
+        account_title = 'Payments'
+        account = Transaction.RECEIVABLE
+        # We use ``Transaction.RECEIVABLE`` which technically counts the number
+        # or orders, not the number of payments.
+        date_periods = self.calculate_date_periods()
+        _, customer_table, customer_extra, _ = \
+            aggregate_transactions_change_by_period(self.provider, account,
+                account_title=account_title,
+                date_periods=date_periods)
+        return customer_table, customer_extra
+
+    def retrieve_metrics(self):
+        customer_table, customer_extra = self.get_data()
+        return Response({
+            "title": "Customers",
+            "results": customer_table,
+            "extra": customer_extra
+        })
+
+
+class PlanMetricsMixin(MetricsMixin):
+
+    def get_data(self):
+        table = []
+        date_periods = self.calculate_date_periods()
+        for plan in Plan.objects.filter(
+                organization=self.provider).order_by('title'):
+            values = active_subscribers_by_period(
+                plan, date_periods=date_periods)
+            table.append({
+                'slug': plan.slug,
+                'title': plan.title,
+                'location': reverse('saas_plan_edit', args=(
+                    self.provider, plan)),
+                'is_active': plan.is_active,
+                'values': values,
+            })
+
+        extra_values = churn_subscribers_by_period(date_periods=date_periods)
+        extra = [{
+            'slug': 'churn',
+            'values': extra_values
+        }]
+        return table, extra
+
+    def retrieve_metrics(self):
+
+        table, extra = self.get_data()
+
+        return Response({
+            'title': _("Active subscribers"),
+            'results': table,
+            'extra': extra
+        })
+
+
+class BalancesAPIView(BalancesMetricsMixin, GenericAPIView):
+    """
+    Retrieves trailing deferred balances
+
+    Generates a table of currency amounts (rows) per period (columns)
+    for a default balance sheet (Income, Backlog, Receivable).
+
+    The date/time returned in `results[].values[]` specifies the end
+    of the period (not included) for which the associated amount
+    in the tuple is computed.
+
+    The date from which trailing balances are computed can be specified
+    by the `ends_at` query parameter. The type of periods (hourly, daily,
+    weekly, monthly, yearly) to aggregate balances over, and the number of
+    periods to return can be specificed by `period_type` and `nb_periods`
+    respectively.
+
+    The API is typically used within an HTML
+    `revenue page </docs/guides/themes/#dashboard_metrics_revenue>`_
+    as present in the default theme.
 
     **Tags**: chart, metrics, provider, transactionmodel
 
@@ -131,33 +336,15 @@ class BalancesAPIView(DateRangeContextMixin, ProviderMixin,
             ]
         }
     """
-    serializer_class = MetricsSerializer
-    filter_backends = (DateRangeFilter,)
 
-    def get(self, request, *args, **kwargs): #pylint: disable=unused-argument
-        result = []
-        unit = settings.DEFAULT_UNIT
-        for key in [Transaction.INCOME, Transaction.BACKLOG,
-                    Transaction.RECEIVABLE]:
-            values, _unit = abs_monthly_balances(
-                organization=self.provider, account=key,
-                until=self.ends_at, tz=self.timezone)
-
-            if _unit:
-                unit = _unit
-
-            result += [{
-                'slug': key,
-                'values': values
-            }]
-        return Response({'title': "Balances",
-            'unit': unit, 'scale': 0.01, 'results': result})
+    @extend_schema(parameters=[QueryParamPeriodSerializer])
+    def get(self, request, *args, **kwargs):
+        return self.retrieve_metrics()
 
 
-class RevenueMetricAPIView(DateRangeContextMixin, ProviderMixin,
-                           GenericAPIView):
+class RevenueMetricAPIView(RevenueMetricsMixin, GenericAPIView):
     """
-    Retrieves 12-month trailing revenue
+    Retrieves trailing revenue
 
     Produces sales, payments and refunds over a period of time.
 
@@ -275,63 +462,10 @@ class RevenueMetricAPIView(DateRangeContextMixin, ProviderMixin,
             ]
         }
     """
-    serializer_class = MetricsSerializer
-    filter_backends = (DateRangeFilter,)
 
+    @extend_schema(parameters=[QueryParamPeriodSerializer])
     def get(self, request, *args, **kwargs):
-        #pylint:disable=unused-argument
-        dates = convert_dates_to_utc(
-            month_periods(12, self.ends_at, tz=self.timezone))
-
-        unit = settings.DEFAULT_UNIT
-
-        account_table, _, _, table_unit = \
-            aggregate_transactions_change_by_period(self.provider,
-                Transaction.RECEIVABLE, account_title='Sales',
-                orig='orig', dest='dest',
-                date_periods=dates)
-
-        _, payment_amounts, payments_unit = aggregate_transactions_by_period(
-            self.provider, Transaction.RECEIVABLE,
-            orig='dest', dest='dest',
-            orig_account=Transaction.BACKLOG,
-            orig_organization=self.provider,
-            date_periods=dates)
-
-        _, refund_amounts, refund_unit = aggregate_transactions_by_period(
-            self.provider, Transaction.REFUND,
-            orig='dest', dest='dest',
-            date_periods=dates)
-
-        units = get_different_units(table_unit, payments_unit, refund_unit)
-
-        if len(units) > 1:
-            LOGGER.error("different units in RevenueMetricAPIView.get: %s",
-                units)
-
-        if units:
-            unit = units[0]
-
-        account_table += [{
-            'slug': "payments",
-            'title': "Payments",
-            'values': payment_amounts
-        }, {
-            'slug': "refunds",
-            'title': "Refunds",
-            'values': refund_amounts
-        }]
-
-        resp = {
-            'title': "Amount",
-            'unit': unit,
-            'scale': 0.01,
-            'results': account_table
-        }
-        if not self.provider.has_bank_account:
-            resp.update({'processor_hint': 'connect_provider'})
-
-        return Response(resp)
+        return self.retrieve_metrics()
 
 
 class CouponUsesQuerysetMixin(object):
@@ -390,10 +524,29 @@ class CouponUsesAPIView(CartItemSmartListMixin, CouponUsesQuerysetMixin,
     serializer_class = CartItemSerializer
 
 
-class CustomerMetricAPIView(DateRangeContextMixin, ProviderMixin,
-                            GenericAPIView):
+class CustomerMetricAPIView(CustomerMetricsMixin, GenericAPIView):
     """
-    Retrieves 12-month trailing customer counts
+    Retrieves trailing customer counts
+
+    Generates a table of total number of customers, number of new customers,
+    number of churned customers, and number of net new customers (rows)
+    per period (columns).
+
+    New customers are defined as customers that made an order in the period,
+    but not in the previous period. Churned customers are defined as customers
+    that made an order in the previous period, but not in the period.
+    The net new customers is defined as the number of new customers minus
+    the number of churned customers.
+
+    The date/time returned in `results[].values[]` specifies the end
+    of the period (not included) for which the associated count
+    in the tuple is computed.
+
+    The date from which trailing balances are computed can be specified
+    by the `ends_at` query parameter. The type of periods (hourly, daily,
+    weekly, monthly, yearly) to aggregate balances over, and the number of
+    periods to return can be specificed by `period_type` and `nb_periods`
+    respectively.
 
     The API is typically used within an HTML
     `revenue page </docs/guides/themes/#dashboard_metrics_revenue>`_
@@ -489,35 +642,17 @@ class CustomerMetricAPIView(DateRangeContextMixin, ProviderMixin,
             ]
         }
     """
-    serializer_class = MetricsSerializer
-    filter_backends = (DateRangeFilter,)
 
+    @extend_schema(parameters=[QueryParamPeriodSerializer])
     def get(self, request, *args, **kwargs):
-        #pylint:disable=unused-argument
-        account_title = 'Payments'
-        account = Transaction.RECEIVABLE
-        # We use ``Transaction.RECEIVABLE`` which technically counts the number
-        # or orders, not the number of payments.
-
-        dates = convert_dates_to_utc(
-            month_periods(12, self.ends_at, tz=self.timezone))
-        _, customer_table, customer_extra, _ = \
-            aggregate_transactions_change_by_period(self.provider, account,
-                account_title=account_title,
-                date_periods=dates)
-
-        return Response({
-            "title": "Customers",
-            "results": customer_table,
-            "extra": customer_extra
-        })
+        return self.retrieve_metrics()
 
 
 class LifetimeValueMetricMixin(DateRangeContextMixin, ProviderMixin):
     """
     Decorates profiles with subscriber age and lifetime value
     """
-    filter_backends = (DateRangeFilter,)
+    filter_backends = (DateRangeFilter,) # XXX includes `DateRangeContextMixin` and `ProviderMixin`
 
     def get_queryset(self):
         organization_model = get_organization_model()
@@ -603,12 +738,25 @@ class LifetimeValueMetricAPIView(LifetimeValueMetricMixin, ListAPIView):
         return self.decorate_queryset(page if page else queryset)
 
 
-class PlanMetricAPIView(DateRangeContextMixin, ProviderMixin, GenericAPIView):
+class PlanMetricAPIView(PlanMetricsMixin, GenericAPIView):
     """
-    Retrieves 12-month trailing plans performance
+    Retrieves trailing plans performance
+
+    Generates a table of active susbribers for each plan (rows)
+    per period (columns).
+
+    The date/time returned in `results[].values[]` specifies the end
+    of the period (not included) for which the associated count
+    in the tuple is computed.
+
+    The date from which trailing balances are computed can be specified
+    by the `ends_at` query parameter. The type of periods (hourly, daily,
+    weekly, monthly, yearly) to aggregate balances over, and the number of
+    periods to return can be specificed by `period_type` and `nb_periods`
+    respectively.
 
     The API is typically used within an HTML
-    `plans metrics page </docs/guides/themes/#dashboard_metrics_plans>`_
+    `revenue page </docs/guides/themes/#dashboard_metrics_revenue>`_
     as present in the default theme.
 
     **Tags**: chart, metrics, provider, planmodel
@@ -692,30 +840,63 @@ class PlanMetricAPIView(DateRangeContextMixin, ProviderMixin, GenericAPIView):
             ]
         }
     """
-    serializer_class = MetricsSerializer
-    filter_backends = (DateRangeFilter,)
 
+    @extend_schema(parameters=[QueryParamPeriodSerializer])
     def get(self, request, *args, **kwargs):
-        #pylint:disable=unused-argument
-        table = []
-        for plan in Plan.objects.filter(
-                organization=self.provider).order_by('title'):
-            values = active_subscribers(
-                plan, from_date=self.ends_at, tz=self.timezone)
-            table.append({
-                'slug': plan.slug,
-                'title': plan.title,
-                'values': values,
-                'location': reverse(
-                    'saas_plan_edit', args=(self.provider, plan)),
-                'is_active': plan.is_active})
-        extra = [{
-            'slug': 'churn',
-            'values': churn_subscribers(
-                from_date=self.ends_at, tz=self.timezone)}]
+        return self.retrieve_metrics()
 
-        return Response({
-            'title': _("Active subscribers"),
-            'results': table,
-            'extra': extra
-        })
+
+class BalancesDueAPIView(BalancesDueMixin, ListAPIView):
+    """
+    Lists subscribers to a provider with a balance due
+
+    This endpoint returns a list of organizations with their respective
+    total contract value, payments made, and total balance due.
+
+    **Tags**: balance, provider, organization, subscribers
+
+    **Examples**
+
+    .. code-block:: http
+
+        GET /api/metrics/cowork/balances-due HTTP/1.1
+
+    responds
+
+    .. code-block:: json
+
+        {
+            "count": 1,
+            "next": null,
+            "previous": null,
+            "results": [
+                {
+                    "slug": "xia",
+                    "printable_name": "Xia Lee",
+                    "picture": null,
+                    "type": "organization",
+                    "credentials": false,
+                    "created_at": "2022-08-31T19:00:00-05:00",
+                    "balances": {
+                        "eur": {
+                            "contract_value": 1333,
+                            "payments": 0,
+                            "balance": 1333
+                        },
+                        "usd": {
+                            "contract_value": 982800,
+                            "cash_payments": 945000,
+                            "balance": 37800
+                        }
+                    }
+                }
+            ]
+        }
+    """
+
+    serializer_class = BalancesDueSerializer
+
+    def paginate_queryset(self, queryset):
+        page = super(BalancesDueAPIView, self).paginate_queryset(
+            queryset)
+        return self.decorate_queryset(page if page else queryset)
