@@ -1,4 +1,4 @@
-# Copyright (c) 2022, DjaoDjin inc.
+# Copyright (c) 2023, DjaoDjin inc.
 # All rights reserved.
 #
 # Redistribution and use in source and binary forms, with or without
@@ -22,7 +22,7 @@
 # OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF
 # ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 
-import hashlib, os, re
+import hashlib, logging, os, re
 
 from dateutil.relativedelta import relativedelta
 from django.conf import settings as django_settings
@@ -39,15 +39,19 @@ from .serializers import (EngagedSubscriberSerializer, OrganizationSerializer,
 from .. import settings, signals
 from ..compat import force_str, gettext_lazy as _, urlparse, urlunparse
 from ..decorators import _valid_manager
-from ..filters import DateRangeFilter, OrderingFilter, SearchFilter
+from ..filters import OrderingFilter, SearchFilter
 from ..mixins import (DateRangeContextMixin, OrganizationMixin,
-    OrganizationSmartListMixin, ProviderMixin, OrganizationDecorateMixin)
+    OrganizationSearchOrderListMixin, OrganizationSmartListMixin,
+    ProviderMixin, OrganizationDecorateMixin)
 from ..models import get_broker
 from ..utils import (build_absolute_uri, datetime_or_now,
     get_organization_model, get_role_model, get_picture_storage,
     handle_uniq_error)
 
 
+LOGGER = logging.getLogger(__name__)
+
+
 class OrganizationQuerysetMixin(OrganizationDecorateMixin):
 
     queryset = get_organization_model().objects.all()
@@ -56,7 +60,7 @@ class OrganizationQuerysetMixin(OrganizationDecorateMixin):
 class OrganizationDetailAPIView(OrganizationMixin, OrganizationQuerysetMixin,
                                 RetrieveUpdateDestroyAPIView):
     """
-    Retrieves a billing profile
+    Retrieves a profile
 
     The API is typically used within an HTML
     `contact information page </docs/guides/themes/#dashboard_profile>`_
@@ -111,7 +115,7 @@ class OrganizationDetailAPIView(OrganizationMixin, OrganizationQuerysetMixin,
 
     def put(self, request, *args, **kwargs):
         """
-        Updates a billing profile
+        Updates a profile
 
         The API is typically used within an HTML
         `contact information page </docs/guides/themes/#dashboard_profile>`_
@@ -156,7 +160,7 @@ class OrganizationDetailAPIView(OrganizationMixin, OrganizationQuerysetMixin,
 
     def delete(self, request, *args, **kwargs):
         """
-        Deletes a billing profile
+        Deletes a profile
 
         We anonymize the profile instead of purely deleting
         it from the database because we don't want to loose history
@@ -272,6 +276,8 @@ class OrganizationPictureAPIView(OrganizationMixin, CreateAPIView):
             hashlib.sha256(uploaded_file.read()).hexdigest(), ext)
         default_storage = get_picture_storage(request)
 
+        LOGGER.debug("upload picture to %s on storage %s",
+            key_name, default_storage)
         location = default_storage.url(
             default_storage.save(key_name, uploaded_file))
         # We are removing the query parameters, as they contain
@@ -290,7 +296,7 @@ class OrganizationListAPIView(OrganizationSmartListMixin,
                               OrganizationQuerysetMixin,
                               ListAPIView):
     """
-    Lists billing profiles
+    Lists profiles
 
     Returns a list of {{PAGE_SIZE}} profile and user accounts.
 
@@ -409,8 +415,6 @@ class EngagedSubscribersSmartListMixin(object):
     """
     reporting entities list which is also searchable and sortable.
     """
-    date_field = 'user__last_login'
-
     search_fields = (
         'first_name',
         'last_name',
@@ -431,11 +435,13 @@ class EngagedSubscribersSmartListMixin(object):
 
     ordering = ('created_at',)
 
-    filter_backends = (DateRangeFilter, SearchFilter, OrderingFilter)
+    filter_backends = (SearchFilter, OrderingFilter,) # No `DateRangeFilter`
+         # because the date range filter is applied as part
+         # of `EngagedSubscribersQuerysetMixin`.
 
 
 class EngagedSubscribersQuerysetMixin(DateRangeContextMixin,
-                                     SubscribersQuerysetMixin):
+                                      SubscribersQuerysetMixin):
 
     def get_queryset(self):
         filter_params = {}
@@ -492,9 +498,10 @@ class EngagedSubscribersAPIView(EngagedSubscribersSmartListMixin,
                   "created_at": "2022-01-01T00:00:00Z",
                   "user": {
                     "slug": "xia23",
-                    "username": "xia23",
-                    "printable_name": "Xia23",
-                    "picture": null
+                    "email": "123@example.com",
+                    "full_name": "",
+                    "created_at": "2022-01-01T00:00:00Z",
+                    "last_login": "2022-01-01T00:00:00Z"
                   },
                   "role_description": {
                     "created_at": "2022-01-01T00:00:00Z",
@@ -506,12 +513,15 @@ class EngagedSubscribersAPIView(EngagedSubscribersSmartListMixin,
                     "profile": null,
                     "extra": null
                   },
+                  "accept_request_api_url": "http://127.0.0.1:8000/api/profile/xia23/roles/manager",
+                  "remove_api_url": "http://127.0.0.1:8000/api/profile/xia23/roles/manager/admin",
                   "profile": {
                     "slug": "xia23",
                     "printable_name": "Xia23",
                     "picture": null,
                     "type": "organization",
-                    "credentials": false
+                    "credentials": false,
+                    "created_at": "2022-01-01T00:00:00Z"
                   }
                 }
             ]
@@ -522,6 +532,8 @@ class EngagedSubscribersAPIView(EngagedSubscribersSmartListMixin,
 
 class UnengagedSubscribersQuerysetMixin(DateRangeContextMixin,
                                        SubscribersQuerysetMixin):
+    # Uses `OrganizationSmartListMixin` here because it derives from
+    # `DateRangeContextMixin` and we need `start_at`/`ends_at` in the query.
 
     def get_queryset(self):
         ends_at = datetime_or_now(self.ends_at)
@@ -542,7 +554,7 @@ class UnengagedSubscribersQuerysetMixin(DateRangeContextMixin,
         return queryset
 
 
-class UnengagedSubscribersAPIView(OrganizationSmartListMixin,
+class UnengagedSubscribersAPIView(OrganizationSearchOrderListMixin,
                                   UnengagedSubscribersQuerysetMixin,
                                   ListAPIView):
     """
@@ -576,13 +588,12 @@ class UnengagedSubscribersAPIView(OrganizationSmartListMixin,
                 {
                 "slug": "xia",
                 "printable_name": "Xia Lee",
-                "created_at": "2016-01-14T23:16:55Z",
-                "ends_at": "2017-01-14T23:16:55Z"
+                "picture": null,
+                "type": "organization",
+                "credentials": false,
+                "created_at": "2016-01-14T23:16:55Z"
                 }
             ]
         }
     """
     serializer_class = OrganizationSerializer
-    filter_backends = (SearchFilter, OrderingFilter) # if we don't remove
-         # the DateRangeFilter, then we will only get unengaged Organization
-         # that were created within the date range.
