@@ -1,4 +1,4 @@
-# Copyright (c) 2022, DjaoDjin inc.
+# Copyright (c) 2023, DjaoDjin inc.
 # All rights reserved.
 #
 # Redistribution and use in source and binary forms, with or without
@@ -32,14 +32,34 @@ from rest_framework.response import Response
 
 from .. import settings
 from ..compat import gettext_lazy as _
-from ..docs import OpenAPIResponse, swagger_auto_schema
-from ..mixins import PlanMixin, CartMixin
-from ..filters import DateRangeFilter, OrderingFilter
+from ..docs import OpenApiResponse, extend_schema
+from ..mixins import CartMixin, DateRangeContextMixin, PlanMixin, ProviderMixin
+from ..filters import (ActiveFilter, DateRangeFilter, OrderingFilter,
+    SearchFilter)
 from ..models import Coupon, Plan, Subscription
 from .serializers import PlanDetailSerializer, PlanCreateSerializer
+from ..utils import datetime_or_now
 
 
-class PricingAPIView(PlanMixin, CartMixin, ListAPIView):
+class PlanSmartListMixin(DateRangeContextMixin, ProviderMixin):
+
+    search_fields = (
+        'slug',
+        'title',
+    )
+
+    ordering_fields = (
+        ('title', 'title'),
+        ('period_amount', 'period_amount'),
+        ('is_active', 'is_active'),
+        ('created_at', 'created_at')
+    )
+    ordering = ('period_amount',)
+
+    filter_backends = (DateRangeFilter, SearchFilter, OrderingFilter,)
+
+
+class PricingAPIView(CartMixin, PlanSmartListMixin, ListAPIView):
     """
     Lists pricing plans
 
@@ -90,34 +110,30 @@ class PricingAPIView(PlanMixin, CartMixin, ListAPIView):
           }]
         }
     """
-    ordering_fields = (
-        ('title', 'title'),
-        ('period_amount', 'period_amount'),
-        ('is_active', 'is_active'),
-        ('created_at', 'created_at'),
-    )
-    ordering = ('period_amount',)
-    filter_backends = (DateRangeFilter, OrderingFilter)
     serializer_class = PlanDetailSerializer
 
     def get_queryset(self):
-        queryset = Plan.objects.filter(organization=self.provider,
-            is_active=True).order_by('is_not_priced', 'period_amount')
+        queryset = Plan.objects.filter(is_active=True).order_by(
+            'is_not_priced', 'period_amount')
+        # `PlanDetailSerializer` will expand `organization`,
+        # `advance_discounts` and `use_charges`.
+        queryset = queryset.select_related('organization').prefetch_related(
+            'advance_discounts').prefetch_related('use_charges')
         return queryset
 
     def paginate_queryset(self, queryset):
+        at_time = datetime_or_now()
         page = super(PricingAPIView, self).paginate_queryset(queryset)
         decorate_queryset = page if page else queryset
 
         cart_items_by_plan = {
             cart_item['plan']: cart_item for cart_item in self.get_cart()}
-
-        redeemed = self.request.session.get('redeemed', None)
-        if redeemed is not None:
-            redeemed = Coupon.objects.active(self.provider, redeemed).first()
+        coupon_code = self.request.session.get('redeemed', None)
 
         #pylint:disable=unused-variable
         for index, plan in enumerate(decorate_queryset):
+            redeemed = Coupon.objects.active(plan.organization, coupon_code,
+                plan=plan, at_time=at_time).first()
             if redeemed and redeemed.is_valid(plan):
                 setattr(plan, 'discounted_period_amount',
                     plan.get_discounted_period_amount(redeemed))
@@ -128,7 +144,7 @@ class PricingAPIView(PlanMixin, CartMixin, ListAPIView):
         return page
 
 
-class PlanListCreateAPIView(PlanMixin, ListCreateAPIView):
+class PlanListCreateAPIView(PlanSmartListMixin, ListCreateAPIView):
     """
     Lists a provider plans
 
@@ -174,23 +190,16 @@ class PlanListCreateAPIView(PlanMixin, ListCreateAPIView):
           }]
         }
     """
-    ordering_fields = (
-        ('title', 'title'),
-        ('period_amount', 'period_amount'),
-        ('is_active', 'is_active'),
-        ('created_at', 'created_at')
-    )
-    ordering = ('period_amount',)
-    filter_backends = (DateRangeFilter, OrderingFilter)
     serializer_class = PlanDetailSerializer
+    filter_backends = (ActiveFilter,) + PlanSmartListMixin.filter_backends
 
     def get_serializer_class(self):
         if self.request.method.lower() == 'post':
             return PlanCreateSerializer
         return super(PlanListCreateAPIView, self).get_serializer_class()
 
-    @swagger_auto_schema(responses={
-      201: OpenAPIResponse("Create successful", PlanDetailSerializer)})
+    @extend_schema(responses={
+      201: OpenApiResponse(PlanDetailSerializer)})
     def post(self, request, *args, **kwargs):
         """
         Creates a plan
@@ -232,11 +241,10 @@ class PlanListCreateAPIView(PlanMixin, ListCreateAPIView):
 
     def get_queryset(self):
         queryset = self.organization.plans.all()
-        is_active = self.request.query_params.get('active')
-        truth_values = ['true', '1']
-        if is_active:
-            value = is_active.lower() in truth_values
-            queryset = queryset.filter(is_active=value)
+        # `PlanDetailSerializer` will expand `organization`,
+        # `advance_discounts` and `use_charges`.
+        queryset = queryset.select_related('organization').prefetch_related(
+            'advance_discounts').prefetch_related('use_charges')
         return queryset
 
     def perform_create(self, serializer):
