@@ -39,7 +39,7 @@ from rest_framework.response import Response
 
 from .. import settings, signals
 from ..compat import force_str, gettext_lazy as _
-from ..docs import OpenAPIResponse, no_body, swagger_auto_schema
+from ..docs import extend_schema, OpenApiResponse
 from ..decorators import _has_valid_access
 from ..filters import OrderingFilter, SearchFilter
 from ..mixins import (OrganizationMixin, OrganizationCreateMixin,
@@ -50,10 +50,11 @@ from ..pagination import RoleListPagination
 from ..utils import (full_name_natural_split, get_organization_model,
     get_role_model, get_role_serializer, generate_random_slug)
 from .organizations import OrganizationDecorateMixin
-from .serializers import (AccessibleSerializer, ForceSerializer,
+from .serializers import (AccessibleSerializer, QueryParamForceSerializer,
     OrganizationCreateSerializer,
     OrganizationDetailSerializer, RoleDescriptionSerializer,
-    AccessibleCreateSerializer, RoleCreateSerializer)
+    AccessibleCreateSerializer, RoleCreateSerializer,
+    QueryParamRoleStatusSerializer, QueryParamPersonalProfSerializer)
 
 
 LOGGER = logging.getLogger(__name__)
@@ -121,16 +122,23 @@ def create_user_from_email(email, password=None, **kwargs):
     return user
 
 
-class OptinBase(OrganizationDecorateMixin, OrganizationCreateMixin):
+class ListOptinAPIView(OrganizationDecorateMixin, OrganizationCreateMixin,
+                       ListCreateAPIView):
 
     organization_model = get_organization_model()
+    serializer_class = AccessibleSerializer
+
+    @property
+    def role_descr(self):
+        return self.kwargs.get('role')
 
     def add_relations(self, organizations, user, ends_at=None):
         #pylint:disable=unused-argument
         roles = []
         created = False
         for organization in organizations:
-            role = organization.add_role_request(user)
+            role = organization.add_role_request(
+                user, role_descr=self.role_descr)
             if role:
                 created = True
                 roles += [role]
@@ -189,7 +197,11 @@ class OptinBase(OrganizationDecorateMixin, OrganizationCreateMixin):
                         organizations = [
                             self.create_organization(organization_data)]
             if not organizations:
-                if not request.GET.get('force', False):
+                query_serializer = QueryParamForceSerializer(
+                    data=self.request.query_params)
+                query_serializer.is_valid(raise_exception=True)
+                force = query_serializer.validated_data.get('force', False)
+                if not force:
                     raise Http404(_("Profile %(organization)s does not exist."
                     ) % {'organization': slug})
                 if not email:
@@ -258,6 +270,7 @@ class InvitedRequestedListMixin(object):
     """
     Filters requests for any role on an organization.
     """
+    role_status_param = 'role_status'
 
     def get_queryset(self):
         queryset = super(InvitedRequestedListMixin, self).get_queryset()
@@ -268,7 +281,12 @@ class InvitedRequestedListMixin(object):
         # here instead of later in RoleInvitedListMixin.
         self.request.invited_count = queryset.filter(
             grant_key__isnull=False).count()
-        role_status = self.request.query_params.get('role_status', '')
+        query_serializer = QueryParamRoleStatusSerializer(
+            data=self.request.query_params)
+        role_status = ''
+        if query_serializer.is_valid(raise_exception=True):
+            role_status = query_serializer.validated_data.get(
+                self.role_status_param, '')
         stts = role_status.split(',')
         flt = None
         if 'active' in stts:
@@ -294,9 +312,24 @@ class InvitedRequestedListMixin(object):
 class AccessibleByQuerysetMixin(UserMixin):
 
     role_model = get_role_model()
+    include_personal_profile_param = 'include_personal_profile'
 
     def get_queryset(self):
-        return self.role_model.objects.filter(user=self.user)
+        queryset = self.role_model.objects.filter(user=self.user)
+        query_serializer = QueryParamPersonalProfSerializer(
+            data=self.request.query_params)
+        query_serializer.is_valid(raise_exception=True)
+
+        include_personal_profile = query_serializer.validated_data.get(
+            self.include_personal_profile_param, False)
+        if not include_personal_profile:
+            queryset = queryset.exclude(organization__slug=self.user)
+
+        # `RoleSerializer` will expand `user` and `role_description`.
+        queryset = queryset.select_related('user').select_related(
+            'role_description')
+
+        return queryset
 
 
 class AccessibleByDescrQuerysetMixin(AccessibleByQuerysetMixin):
@@ -306,9 +339,10 @@ class AccessibleByDescrQuerysetMixin(AccessibleByQuerysetMixin):
             ).filter(role_description__slug=self.kwargs.get('role'))
 
 
-class AccessibleByListAPIView(RoleSmartListMixin, InvitedRequestedListMixin,
+class AccessibleByListAPIView(RoleSmartListMixin,
+                              InvitedRequestedListMixin,
                               AccessibleByQuerysetMixin,
-                              OptinBase, ListCreateAPIView):
+                              ListOptinAPIView):
     """
     Lists roles by user
 
@@ -384,9 +418,8 @@ accessibles/manager/cowork",
             return AccessibleCreateSerializer
         return super(AccessibleByListAPIView, self).get_serializer_class()
 
-    @swagger_auto_schema(responses={
-      201: OpenAPIResponse("Create successful", AccessibleSerializer)},
-        query_serializer=ForceSerializer)
+    @extend_schema(parameters=[QueryParamForceSerializer], responses={
+      201: OpenApiResponse(AccessibleSerializer)})
     def post(self, request, *args, **kwargs):
         """
         Requests a role
@@ -446,8 +479,8 @@ accessibles/manager/cowork",
 
 class AccessibleByDescrListAPIView(RoleSmartListMixin,
                                    InvitedRequestedListMixin,
-                                   AccessibleByDescrQuerysetMixin, UserMixin,
-                                   ListCreateAPIView):
+                                   AccessibleByDescrQuerysetMixin,
+                                   ListOptinAPIView):
     """
     Lists roles of specific type by user
 
@@ -521,9 +554,14 @@ accessibles/manager/cowork",
             return AccessibleCreateSerializer
         return super(AccessibleByDescrListAPIView, self).get_serializer_class()
 
-    @swagger_auto_schema(responses={
-      201: OpenAPIResponse("Create successful", AccessibleSerializer)},
-        query_serializer=ForceSerializer)
+    @extend_schema(operation_id='users_accessibles_list_by_role')
+    def get(self, request, *args, **kwargs):
+        return super(AccessibleByDescrListAPIView, self).get(
+            request, *args, **kwargs)
+
+    @extend_schema(operation_id='users_accessibles_create_by_role',
+        parameters=[QueryParamForceSerializer], responses={
+        201: OpenApiResponse(AccessibleSerializer)})
     def post(self, request, *args, **kwargs): #pylint:disable=unused-argument
         """
         Requests a role of a specified type
@@ -588,6 +626,21 @@ accessibles/manager/cowork",
         return self.perform_optin(serializer, request, user=self.user)
 
 
+class RoleDescriptionSmartListMixin(object):
+
+    search_fields = (
+        'slug',
+        'title',
+    )
+    ordering_fields = (
+        ('slug', 'slug'),
+        ('title', 'title'),
+    )
+    ordering = ('slug',)
+
+    filter_backends = (SearchFilter, OrderingFilter)
+
+
 class RoleDescriptionQuerysetMixin(OrganizationMixin):
 
     def get_queryset(self):
@@ -599,7 +652,8 @@ class RoleDescriptionQuerysetMixin(OrganizationMixin):
             raise PermissionDenied()
 
 
-class RoleDescriptionListCreateView(RoleDescriptionQuerysetMixin,
+class RoleDescriptionListCreateView(RoleDescriptionSmartListMixin,
+                                    RoleDescriptionQuerysetMixin,
                                     ListCreateAPIView):
     """
     Lists role types
@@ -652,18 +706,6 @@ class RoleDescriptionListCreateView(RoleDescriptionQuerysetMixin,
             ]
         }
     """
-    search_fields = (
-        'slug',
-        'title',
-    )
-    ordering_fields = (
-        ('slug', 'slug'),
-        ('title', 'title'),
-    )
-    ordering = ('slug',)
-
-    filter_backends = (SearchFilter, OrderingFilter)
-
     serializer_class = RoleDescriptionSerializer
 
     def post(self, request, *args, **kwargs):
@@ -824,7 +866,12 @@ class RoleDescriptionDetailView(RoleDescriptionQuerysetMixin,
 class RoleQuerysetBaseMixin(OrganizationMixin):
 
     def get_queryset(self):
-        return get_role_model().objects.filter(organization=self.organization)
+        queryset = get_role_model().objects.filter(
+            organization=self.organization)
+        # `RoleSerializer` will expand `user` and `role_description`.
+        queryset = queryset.select_related('user').select_related(
+            'role_description')
+        return queryset
 
 
 class RoleListAPIView(RoleSmartListMixin, InvitedRequestedListMixin,
@@ -887,8 +934,15 @@ class RoleByDescrQuerysetMixin(RoleDescriptionMixin, RoleQuerysetBaseMixin):
     # will be None on requested roles. We thus need a `role_description = X
     # *OR* request_key IS NOT NULL`. Calls to more and more refined `filter`
     # through class inheritence only allows us to implement `*AND*`.
+    role_status_param = 'role_status'
 
     def get_queryset(self):
+        query_serializer = QueryParamRoleStatusSerializer(
+            data=self.request.query_params)
+        query_serializer.is_valid(raise_exception=True)
+        role_status = query_serializer.validated_data.get(
+            self.role_status_param, '')
+
         queryset = super(RoleByDescrQuerysetMixin, self).get_queryset()
         self.request.requested_count = queryset.filter(
             request_key__isnull=False).count()
@@ -898,7 +952,7 @@ class RoleByDescrQuerysetMixin(RoleDescriptionMixin, RoleQuerysetBaseMixin):
         self.request.invited_count = queryset.filter(
             role_description=self.role_description,
             grant_key__isnull=False).count()
-        role_status = self.request.query_params.get('role_status', '')
+
         stts = role_status.split(',')
         flt = (Q(role_description=self.role_description) |
             Q(request_key__isnull=False))
@@ -999,7 +1053,11 @@ class RoleByDescrListAPIView(RoleSmartListMixin, RoleByDescrQuerysetMixin,
             except user_model.DoesNotExist:
                 user = None
         if not user:
-            if not request.GET.get('force', False):
+            query_serializer = QueryParamForceSerializer(
+                data=self.request.query_params)
+            query_serializer.is_valid(raise_exception=True)
+            force = query_serializer.validated_data.get('force', False)
+            if not force:
                 sep = ""
                 not_found_msg = "Cannot find"
                 if serializer.validated_data.get('slug'):
@@ -1033,9 +1091,13 @@ class RoleByDescrListAPIView(RoleSmartListMixin, RoleByDescrQuerysetMixin,
         return Response(serializer.validated_data, status=resp_status,
             headers=self.get_success_headers(serializer.validated_data))
 
-    @swagger_auto_schema(responses={
-      201: OpenAPIResponse("Create successful", get_role_serializer())},
-        query_serializer=ForceSerializer)
+    @extend_schema(operation_id='profile_roles_list_by_role')
+    def get(self, request, *args, **kwargs):
+        return super(RoleByDescrListAPIView, self).get(
+            request, *args, **kwargs)
+
+    @extend_schema(parameters=[QueryParamForceSerializer], responses={
+      201: OpenApiResponse(get_role_serializer())})
     def post(self, request, *args, **kwargs):
         """
         Creates a role
@@ -1090,7 +1152,7 @@ class RoleDetailAPIView(RoleMixin, DestroyAPIView):
     """
     serializer_class = get_role_serializer()
 
-    @swagger_auto_schema(request_body=no_body)
+    @extend_schema(operation_id='profile_roles_invite', request=None)
     def post(self, request, *args, **kwargs):#pylint:disable=unused-argument
         """
         Sends invite notification for a role
@@ -1155,8 +1217,11 @@ class RoleDetailAPIView(RoleMixin, DestroyAPIView):
         return super(RoleDetailAPIView, self).delete(request, *args, **kwargs)
 
     def destroy(self, request, *args, **kwargs):
+        self.kwargs.pop('role') # If we leave 'role' in kwargs, we can't deny
+                                # requests that haven't been accepted yet.
         queryset = self.get_queryset()
-        roles = [str(role.role_description) for role in queryset]
+        roles = [str(role.role_description) for role in queryset
+            if role.role_description]
         LOGGER.info("Remove roles %s for user '%s' on organization '%s'",
             roles, self.user, self.organization,
             extra={'event': 'remove-roles', 'user': self.user,
@@ -1174,10 +1239,14 @@ class AccessibleDetailAPIView(RoleDetailAPIView):
     def get_queryset(self):
         # We never take the `role_description` into account when removing
         # on the accessibles page.
-        return get_role_model().objects.filter(
+        queryset = get_role_model().objects.filter(
             organization=self.organization, user=self.user)
+        # `RoleSerializer` will expand `user` and `role_description`.
+        queryset = queryset.select_related('user').select_related(
+            'role_description')
+        return queryset
 
-    @swagger_auto_schema(request_body=no_body)
+    @extend_schema(operation_id='users_accessibles_invite', request=None)
     def post(self, request, *args, **kwargs):
         """
         Sends request notification for role
@@ -1249,7 +1318,7 @@ class RoleAcceptAPIView(UserMixin, GenericAPIView):
 
     serializer_class = AccessibleSerializer
 
-    @swagger_auto_schema(request_body=no_body)
+    @extend_schema(request=None)
     def put(self, request, *args, **kwargs):#pylint:disable=unused-argument
         """
         Accepts role invite
@@ -1372,6 +1441,7 @@ class UserProfileListAPIView(OrganizationSmartListMixin,
         }
     """
     serializer_class = OrganizationDetailSerializer
+    convert_from_personal_param = 'convert_from_personal'
 
     def paginate_queryset(self, queryset):
         page = super(UserProfileListAPIView, self).paginate_queryset(queryset)
@@ -1384,11 +1454,13 @@ class UserProfileListAPIView(OrganizationSmartListMixin,
         return super(UserProfileListAPIView, self).get_serializer_class()
 
     def get_queryset(self):
-        return get_organization_model().objects.accessible_by(
+        queryset = get_organization_model().objects.accessible_by(
             self.user, role_descr=settings.MANAGER)
+        return queryset
 
-    @swagger_auto_schema(responses={
-      201: OpenAPIResponse("Create successful", OrganizationDetailSerializer)})
+
+    @extend_schema(responses={
+      201: OpenApiResponse(OrganizationDetailSerializer)})
     def post(self, request, *args, **kwargs):
         """
         Creates a connected profile
@@ -1427,11 +1499,38 @@ class UserProfileListAPIView(OrganizationSmartListMixin,
         return super(UserProfileListAPIView, self).post(
             request, *args, **kwargs)
 
+    def is_valid_convert_to_organization_request(self, organization):
+        return organization and organization.attached_user() == self.user
+
     def create(self, request, *args, **kwargs): #pylint:disable=unused-argument
         #pylint:disable=unused-argument
         serializer = self.get_serializer(data=request.data)
         serializer.is_valid(raise_exception=True)
         validated_data = dict(serializer.validated_data)
+
+        # If we're creating an organization from a personal profile
+        query_serializer = QueryParamPersonalProfSerializer(
+            data=request.query_params)
+        query_serializer.is_valid(raise_exception=True)
+        convert_from_personal = query_serializer.validated_data.get(
+            self.convert_from_personal_param, False)
+
+        if convert_from_personal:
+            organization = self.get_queryset().filter(
+                slug__exact=self.user.username).first()
+
+            if not self.is_valid_convert_to_organization_request(organization):
+                return Response({'error': _('Invalid request.')},
+                    status=status.HTTP_400_BAD_REQUEST)
+
+            organization.full_name = serializer.validated_data.get('full_name')
+            organization.slug = serializer.validated_data.get('slug', None)
+            organization.save()
+
+            return Response({"detail":
+                _("Successfully converted personal profile to organization.")},
+                status=status.HTTP_200_OK)
+
         if 'email' not in validated_data:
             # email is optional to create the profile but it is required
             # to save the record in the database.
@@ -1448,8 +1547,8 @@ class UserProfileListAPIView(OrganizationSmartListMixin,
         self.decorate_personal(organization)
 
         # returns created profile
-        serializer = super(UserProfileListAPIView, self).get_serializer_class()(
-            instance=organization)
+        serializer = self.serializer_class(
+            instance=organization, context=self.get_serializer_context())
         headers = self.get_success_headers(serializer.data)
         return Response(serializer.data,
             status=status.HTTP_201_CREATED, headers=headers)
