@@ -35,7 +35,7 @@ granted access to personal information.
 """
 from __future__ import unicode_literals
 
-import logging
+import json, logging
 
 from django.core import validators
 from django.contrib.auth import get_user_model
@@ -51,10 +51,11 @@ import phonenumbers
 from .. import settings
 from ..compat import gettext_lazy as _, reverse, six
 from ..decorators import _valid_manager
-from ..humanize import as_money
-from ..mixins import as_html_description, product_url
-from ..models import (AdvanceDiscount, BalanceLine, CartItem, Charge, Coupon,
-    Plan, RoleDescription, Subscription, Transaction)
+from ..humanize import MONTHLY, as_money
+from ..mixins import as_html_description, product_url, read_agreement_file
+from ..models import (get_broker, AdvanceDiscount, Agreement, BalanceLine,
+    CartItem, Charge, Coupon, Plan, RoleDescription, Subscription, Transaction,
+    UseCharge)
 from ..utils import (build_absolute_uri, get_organization_model, get_role_model,
     get_user_serializer, get_user_detail_serializer)
 
@@ -62,39 +63,38 @@ from ..utils import (build_absolute_uri, get_organization_model, get_role_model,
 LOGGER = logging.getLogger(__name__)
 
 
-class EnumField(serializers.Field):
+class EnumField(serializers.ChoiceField):
     """
     Treat a ``PositiveSmallIntegerField`` as an enum.
     """
-    choices = {}
-    inverted_choices = {}
+    translated_choices = {}
 
     def __init__(self, choices, *args, **kwargs):
-        self.choices = dict(choices)
-        self.inverted_choices = {
-            slugify(val): key for key, val in six.iteritems(self.choices)}
-        super(EnumField, self).__init__(*args, **kwargs)
+        self.translated_choices = {key: slugify(val) for key, val in choices}
+        super(EnumField, self).__init__([(slugify(val), key)
+            for key, val in choices],
+            *args, **kwargs)
 
     def to_representation(self, value):
         if isinstance(value, list):
-            result = [slugify(self.choices.get(item, None))
+            result = [slugify(self.translated_choices.get(item, None))
                 for item in value]
         else:
-            result = slugify(self.choices.get(value, None))
+            result = slugify(self.translated_choices.get(value, None))
         return result
 
     def to_internal_value(self, data):
         if isinstance(data, list):
-            result = [self.inverted_choices.get(item, None) for item in data]
+            result = [self.choices.get(item, None) for item in data]
         else:
-            result = self.inverted_choices.get(data, None)
+            result = self.choices.get(data, None)
         if result is None:
             if not data:
                 raise ValidationError(_("This field cannot be blank."))
             raise ValidationError(_("'%(data)s' is not a valid choice."\
                 " Expected one of %(choices)s.") % {
                     'data': data, 'choices': [str(choice)
-                    for choice in six.iterkeys(self.inverted_choices)]})
+                    for choice in six.iterkeys(self.choices)]})
         return result
 
 
@@ -158,6 +158,19 @@ class PlanRelatedField(serializers.RelatedField):
         return get_object_or_404(Plan.objects.all(), slug=data)
 
 
+class UseChargeRelatedField(serializers.RelatedField):
+
+    def __init__(self, **kwargs):
+        super(UseChargeRelatedField, self).__init__(
+            queryset=UseCharge.objects.all(), **kwargs)
+
+    def to_representation(self, value):
+        return value.slug
+
+    def to_internal_value(self, data):
+        return get_object_or_404(UseCharge.objects.all(), slug=data)
+
+
 class RoleDescriptionRelatedField(serializers.RelatedField):
 
     def __init__(self, **kwargs):
@@ -262,7 +275,7 @@ class CardTokenSerializer(NoModelSerializer):
     postal_code = serializers.CharField(required=False, allow_blank=True,
         help_text=_("Zip/Postal code"))
     country = CountryField(required=False, allow_blank=True,
-        help_text=_("Country"))
+        help_text=_("Country as 2-letter code (ISO 3166-1)"))
 
 
 class ChargeSerializer(serializers.ModelSerializer):
@@ -368,13 +381,6 @@ class EmailChargeReceiptSerializer(NoModelSerializer):
         help_text=_("Feedback for the user in plain text"))
 
 
-class ForceSerializer(NoModelSerializer):
-
-    force = serializers.BooleanField(required=False,
-        help_text=_("Forces invite of user/organization that could"\
-        " not be found"))
-
-
 class DatetimeValueTuple(serializers.ListField):
 
     child = serializers.CharField() # XXX (Datetime, Integer)
@@ -434,17 +440,6 @@ class RefundChargeSerializer(NoModelSerializer):
         help_text=_("Line items in a charge to be refunded"))
 
 
-class OrganizationBalanceSerializer(NoModelSerializer):
-
-    balance_amount = serializers.IntegerField(read_only=True,
-      help_text=_("balance of all transactions in cents (i.e. 100ths) of unit"))
-    balance_unit = serializers.CharField(read_only=True,
-        help_text=_("three-letter ISO 4217 code for currency unit (ex: usd)"))
-
-    class Meta:
-        fields = ('balance_amount', 'balance_unit')
-
-
 class WithEndsAtByPlanSerializer(NoModelSerializer):
 
     plan = serializers.SlugField(source='plan__slug', read_only=True)
@@ -529,10 +524,10 @@ class OrganizationDetailSerializer(OrganizationSerializer):
     postal_code = serializers.CharField(required=False, allow_blank=True,
         help_text=_("Zip/Postal code"))
     country = CountryField(required=False, allow_blank=True,
-        help_text=_("Country"))
+        help_text=_("Country as 2-letter code (ISO 3166-1)"))
     is_bulk_buyer = serializers.BooleanField(required=False, default=False,
         help_text=_("Enable GroupBuy"))
-    extra = serializers.CharField(required=False, allow_null=True,
+    extra = ExtraField(required=False, allow_null=True,
         help_text=_("Extra meta data (can be stringify JSON)"))
 
     detail = serializers.CharField(read_only=True, required=False,
@@ -608,6 +603,24 @@ class AdvanceDiscountSerializer(serializers.ModelSerializer):
         fields = ('discount_type', 'discount_value', 'length')
 
 
+class UseChargeSerializer(serializers.ModelSerializer):
+
+    class Meta:
+        model = UseCharge
+        fields = ('slug', 'title', 'description', 'created_at',
+            'use_amount', 'quota', 'maximum_limit', 'extra')
+
+    def validate(self, attrs):
+        maximum_limit = attrs.get('maximum_limit')
+        quota = attrs.get('quota')
+
+        if maximum_limit is not None and maximum_limit != 0:
+            if quota >= maximum_limit:
+                raise serializers.ValidationError(
+                    {"error": "Maximum limit must be greater than the quota."})
+        return attrs
+
+
 class PlanDetailSerializer(PlanSerializer):
 
     description = serializers.CharField(required=False,
@@ -636,7 +649,8 @@ class PlanDetailSerializer(PlanSerializer):
         " to the plan initiated by a subscriber. (defaults to False)"))
     advance_discounts = AdvanceDiscountSerializer(many=True, required=False,
         help_text=_("Discounts when periods are paid in advance."))
-
+    use_charges = UseChargeSerializer(many=True, required=False,
+        help_text=_("Variable pricing based on usage"))
     discounted_period_amount = serializers.SerializerMethodField(required=False,
         help_text=_("Discounted amount for the first period"))
     is_cart_item = serializers.SerializerMethodField(required=False,
@@ -650,7 +664,7 @@ class PlanDetailSerializer(PlanSerializer):
         model = PlanSerializer.Meta.model
         fields = PlanSerializer.Meta.fields + ('description', 'is_active',
             'setup_amount', 'period_amount', 'period_type',
-            'advance_discounts', 'unit', 'profile', 'extra',
+            'advance_discounts', 'use_charges', 'unit', 'profile', 'extra',
             'period_length', 'renewal_type', 'is_not_priced', 'unlock_event',
             'created_at', 'skip_optin_on_grant', 'optin_on_request',
             'discounted_period_amount', 'is_cart_item', 'detail')
@@ -667,6 +681,7 @@ class PlanDetailSerializer(PlanSerializer):
 
     def create(self, validated_data):
         advance_discounts = validated_data.pop('advance_discounts', [])
+        use_charges = validated_data.pop('use_charges', [])
         with transaction.atomic():
             instance = Plan.objects.create(**validated_data)
             for advance_discount in advance_discounts:
@@ -675,11 +690,22 @@ class PlanDetailSerializer(PlanSerializer):
                     discount_type=advance_discount.get('discount_type'),
                     discount_value=advance_discount.get('discount_value'),
                     length=advance_discount.get('length'))
+            for use_charge in use_charges:
+                UseCharge.objects.create(
+                    plan=instance,
+                    slug=use_charge.get('slug'),
+                    title=use_charge.get('title'),
+                    description=use_charge.get('description', ""),
+                    use_amount=use_charge.get('use_amount', 0),
+                    quota=use_charge.get('quota', 0),
+                    maximum_limit=use_charge.get('maximum_limit'),
+                    extra=use_charge.get('extra'))
 
         return instance
 
     def update(self, instance, validated_data):
         advance_discounts = validated_data.pop('advance_discounts', [])
+        use_charges = validated_data.pop('use_charges', [])
         for attr, value in six.iteritems(validated_data):
             setattr(instance, attr, value)
 
@@ -693,6 +719,21 @@ class PlanDetailSerializer(PlanSerializer):
                     discount_value=advance_discount.get('discount_value'),
                     length=advance_discount.get('length'))
 
+            instance.use_charges.exclude(slug__in=[
+                use_charge.get('slug') for use_charge in use_charges]).delete()
+            for use_charge in use_charges:
+                use_charge_slug = use_charge.get('slug')
+                UseCharge.objects.update_or_create(
+                    defaults={
+                        'title': use_charge.get('title'),
+                        'description': use_charge.get('description'),
+                        'use_amount': use_charge.get('use_amount'),
+                        'quota': use_charge.get('quota'),
+                        'maximum_limit': use_charge.get('maximum_limit'),
+                        'extra': use_charge.get('extra')
+                    },
+                    plan=instance, slug=use_charge_slug)
+
         return instance
 
 
@@ -706,6 +747,20 @@ class PlanCreateSerializer(PlanDetailSerializer):
     class Meta(PlanDetailSerializer.Meta):
         fields = PlanDetailSerializer.Meta.fields
 
+    def validate(self, attrs):
+        period_type = attrs.get('period_type',
+            Plan.INTERVAL_CHOICES[MONTHLY - 1][1])
+        period_amount = attrs.get('period_amount', 0)
+
+        min_amount = getattr(settings, 'BROKER_MINIMUM_PLAN_AMOUNT_%s' %
+            period_type, 0)
+        if min_amount and period_amount < min_amount:
+            raise ValidationError(
+                _('Period amount must be greater or equal to %(min_amount)s.')
+                % {'min_amount': min_amount})
+
+        return attrs
+
 
 class OrganizationInviteSerializer(OrganizationCreateSerializer):
 
@@ -723,6 +778,13 @@ class OrganizationInviteSerializer(OrganizationCreateSerializer):
 
 
 class SubscriptionSerializer(serializers.ModelSerializer):
+    """
+    Base class to serialize `Subscription` instances.
+
+    The actual serializer used in API endpoints will either
+    be `SubscribedSubscriptionSerializer` for subscriber-side APIs
+    or `ProvidedSubscriptionSerializer` for provider-side APIs.
+    """
 
     # XXX used to be OrganizationDetailSerializer. because it is used
     # in checkout with group buy, do we need the e-mail?
@@ -731,9 +793,6 @@ class SubscriptionSerializer(serializers.ModelSerializer):
         help_text=_("Profile subscribed to the plan"))
     plan = PlanSerializer(read_only=True,
         help_text=_("Plan the profile is subscribed to"))
-    editable = serializers.SerializerMethodField(
-        help_text=_("True if the request user is able to update"\
-        " the subscription. Typically a manager for the plan provider."))
     app_url = serializers.SerializerMethodField(
       help_text=_("URL to access the subscribed service"))
 
@@ -742,7 +801,7 @@ class SubscriptionSerializer(serializers.ModelSerializer):
         fields = ('created_at', 'ends_at', 'description',
                   'profile', 'plan', 'auto_renew',
                   'extra', 'grant_key', 'request_key',
-                  'editable', 'app_url')
+                  'app_url')
         # XXX grant_key and request_key should probably be removed.
         read_only_fields = ('grant_key', 'request_key')
 
@@ -750,20 +809,26 @@ class SubscriptionSerializer(serializers.ModelSerializer):
         return product_url(subscriber=obj.organization, plan=obj.plan,
             request=self.context['request'])
 
-    def get_editable(self, subscription):
-        return bool(_valid_manager(self.context['request'],
-            [subscription.plan.organization]))
-
 
 class SubscribedSubscriptionSerializer(SubscriptionSerializer):
 
     remove_api_url = serializers.SerializerMethodField(
-        help_text=_("URL API endpoint to remove the subscription"\
-        " grant or request"))
+        help_text=_("URL API endpoint to remove the subscription request"))
+
+    # `editable` is only subscribed-side since all APIs provider-side
+    # will by definition return subscribers / subscriptions a profile manager
+    # for the plan provider has permission to.
+    editable = serializers.SerializerMethodField(
+        help_text=_("True if the request user is able to update the"\
+        " subscription. Typically a profile manager for the plan provider."))
 
     class Meta(SubscriptionSerializer.Meta):
         fields = SubscriptionSerializer.Meta.fields + (
-            'remove_api_url',)
+            'remove_api_url', 'editable')
+
+    def get_editable(self, subscription):
+        return bool(_valid_manager(self.context['request'],
+            [subscription.plan.organization]))
 
     def get_remove_api_url(self, obj):
         return build_absolute_uri(self.context['request'], location=reverse(
@@ -776,8 +841,7 @@ class ProvidedSubscriptionSerializer(SubscriptionSerializer):
     accept_request_api_url = serializers.SerializerMethodField(
         help_text=_("URL API endpoint to grant the subscription"))
     remove_api_url = serializers.SerializerMethodField(
-        help_text=_("URL API endpoint to remove the subscription"\
-        " grant or request"))
+        help_text=_("URL API endpoint to remove the subscription grant"))
 
     class Meta(SubscriptionSerializer.Meta):
         fields = SubscriptionSerializer.Meta.fields + (
@@ -891,7 +955,7 @@ class CreateOfflineTransactionSerializer(NoModelSerializer):
     subscription = serializers.CharField(
         help_text="The subscription the offline transaction refers to.")
     created_at = serializers.DateTimeField(
-        help_text=_("Date/time of creation (in ISO format)"))
+        help_text=_("Date/time of creation (in ISO 8601 format)"))
     # XXX Shouldn't this be same format as TransactionSerializer.amount?
     amount = serializers.DecimalField(None, 2,
         help_text=_("Total amount in currency unit"))
@@ -927,9 +991,10 @@ class CartItemSerializer(serializers.ModelSerializer):
 
     class Meta:
         model = CartItem
-        fields = ('created_at', 'user', 'plan', 'option', 'full_name',
-            'sync_on', 'email', 'detail')
-        read_only_fields = ('created_at', 'user', 'detail')
+        fields = ('created_at', 'user', 'claim_code',
+            'plan', 'option', 'use', 'quantity',
+            'sync_on', 'full_name', 'email', 'detail')
+        read_only_fields = ('created_at', 'user', 'claim_code', 'detail')
 
     @staticmethod
     def get_detail(obj):
@@ -940,19 +1005,51 @@ class CartItemSerializer(serializers.ModelSerializer):
         return None
 
 
+class CartItemUpdateSerializer(CartItemSerializer):
+    """
+    Designed for handling update operations on cart items.
+    Restricts user and plan fields to be read-only.
+    """
+    user = get_user_serializer()(
+        help_text=_("User the cart belongs to"), read_only=True)
+    plan = PlanSerializer(
+        help_text=_("Plan in the cart (if any)"), read_only=True)
+
+    class Meta(CartItemSerializer.Meta):
+        read_only_fields = ('plan',) + CartItemSerializer.Meta.read_only_fields
+
+
 class CartItemCreateSerializer(serializers.ModelSerializer):
     """
     Serializer to build a request.user set of plans to subscribe to (i.e. cart).
     """
     plan = PlanRelatedField(read_only=False, required=True,
         help_text=_("The plan to add into the request.user cart."))
+    use = UseChargeRelatedField(required=False,
+        help_text=_("The use charge to add into the request.user cart."))
 
     class Meta:
         model = CartItem
-        fields = ('created_at', 'plan', 'option', 'full_name',
-            'sync_on', 'email')
+        fields = ('created_at', 'plan', 'option',
+                  'use', 'quantity',
+                  'sync_on', 'full_name', 'email')
         read_only_fields = ('created_at',)
 
+class UserCartItemCreateSerializer(CartItemCreateSerializer):
+    """
+    Extends `CartItemCreateSerializer` to include user.
+    It's used during the creation of new cart items to
+    ensure necessary data is captured.
+    """
+    user = serializers.SlugRelatedField(
+        queryset=get_user_model().objects.all(),
+        slug_field='username',
+        required=True,
+        help_text=_('The user for whom the cart item is being created')
+    )
+
+    class Meta(CartItemCreateSerializer.Meta):
+        fields = ('user',) + CartItemCreateSerializer.Meta.fields
 
 class CartItemUploadSerializer(NoModelSerializer):
 
@@ -983,12 +1080,37 @@ class RoleDescriptionSerializer(serializers.ModelSerializer):
     is_global = serializers.BooleanField(read_only=True,
         help_text=_("True when the role type is available for all profiles"))
 
+    # `editable` means the request user can update / delete the role
+    # description.
+    editable = serializers.SerializerMethodField(
+        help_text=_("True if the request user is able to update the"\
+        " role description. Typically a profile manager"\
+        " for the organization (local role description) or the broker"\
+        " (global role descriptions)."))
+
     class Meta:
         model = RoleDescription
         fields = ('created_at', 'slug', 'title',
                   'skip_optin_on_grant', 'implicit_create_on_none',
-                  'is_global', 'profile', 'extra')
-        read_only_fields = ('created_at', 'slug', 'is_global')
+                  'otp_required', 'is_global', 'profile', 'extra', 'editable')
+        read_only_fields = ('created_at', 'slug', 'is_global', 'editable')
+
+    def get_editable(self, obj):
+        candidates = [get_broker()]
+        try:
+            if obj.organization:
+                candidates += [obj.organization]
+        except AttributeError:
+            # serializers for notification will use this serializer,
+            # though passing dictionnaries instead of an object.
+            # the `editable` field was never intended for notifications
+            # so it is ok to ignore here.
+            pass
+        # In notifications (triggered by signals), we will not have
+        # `request` in the context either.
+        if 'request' in self.context:
+            return bool(_valid_manager(self.context['request'], candidates))
+        return False
 
 
 class AccessibleSerializer(serializers.ModelSerializer):
@@ -1156,12 +1278,58 @@ class UploadBlobSerializer(NoModelSerializer):
         help_text=_("URL to uploaded content"))
 
 
+class AgreementSerializer(serializers.ModelSerializer):
+
+    class Meta:
+        model = Agreement
+        fields = ('slug', 'title', 'updated_at')
+        read_only_fields = ('slug', 'updated_at')
+
+
+class AgreementDetailSerializer(AgreementSerializer):
+    """
+    Serializer to retrieve the text of an agreement
+    """
+    text = serializers.SerializerMethodField(
+        help_text=_("Text of the agreement"))
+
+    def get_text(self, obj):
+        return read_agreement_file(obj.slug, request=self.context['request'])
+
+    class Meta(AgreementSerializer.Meta):
+        fields = AgreementSerializer.Meta.fields + ('text',)
+
+
+class AgreementUpdateSerializer(AgreementSerializer):
+    """
+    Serializer to update an agreement
+    """
+    updated_at = serializers.DateTimeField(read_only=False,
+        help_text=_("Date/time of update (in ISO 8601 format)"))
+
+    class Meta(AgreementSerializer.Meta):
+        model = AgreementSerializer.Meta.model
+        fields = AgreementSerializer.Meta.fields
+
+
+class AgreementCreateSerializer(AgreementUpdateSerializer):
+    """
+    Serializer to create an agreement
+    """
+    updated_at = serializers.DateTimeField(required=False,
+        help_text=_("Date/time of update (in ISO 8601 format)"))
+
+    class Meta(AgreementUpdateSerializer.Meta):
+        model = AgreementUpdateSerializer.Meta.model
+        fields = AgreementUpdateSerializer.Meta.fields
+
+
 class AgreementSignSerializer(NoModelSerializer):
 
     read_terms = serializers.BooleanField(required=True,
         help_text=_("I have read and understand these terms and conditions"))
     last_signed = serializers.DateTimeField(read_only=True,
-        help_text=_("Date/time of signature (in ISO format)"))
+        help_text=_("Date/time of signature (in ISO 8601 format)"))
 
     class Meta:
         fields = ('read_terms', 'last_signed',)
@@ -1236,8 +1404,8 @@ class CheckoutSerializer(NoModelSerializer):
         help_text=_("State/Province/County"))
     postal_code = serializers.CharField(required=False, allow_blank=True,
         help_text=_("Zip/Postal code"))
-    country = serializers.CharField(required=False, allow_blank=True,
-        help_text=_("Country"))
+    country = CountryField(required=False, allow_blank=True,
+        help_text=_("Country as 2-letter code (ISO 3166-1)"))
 
 
 class RedeemCouponSerializer(NoModelSerializer):
@@ -1249,3 +1417,76 @@ class RedeemCouponSerializer(NoModelSerializer):
 
     def create(self, validated_data):
         return validated_data
+
+
+class BalancesDueSerializer(OrganizationSerializer):
+
+    balances = serializers.DictField(
+        help_text=_("Dictionary of balances due, keyed by unit"),
+        read_only=True)
+
+    class Meta(OrganizationSerializer.Meta):
+        fields = OrganizationSerializer.Meta.fields + ('balances',)
+
+
+# Serializers to document HTTP query parameters
+
+class QueryParamActiveSerializer(NoModelSerializer):
+
+    active = serializers.BooleanField(required=False,
+        help_text=_("True when customers can subscribe to the plan"),
+        default=None, allow_null=True)
+
+
+class QueryParamCartItemSerializer(NoModelSerializer):
+
+    plan = PlanRelatedField(required=False, allow_null=True,
+            help_text=_("Plan"))
+    email = serializers.EmailField(required=False,
+        help_text=_("E-mail address"), default=None, allow_null=True)
+
+
+class QueryParamForceSerializer(NoModelSerializer):
+
+    force = serializers.BooleanField(required=False,
+        help_text=_("Forces invite of user/organization that could"\
+        " not be found"))
+
+
+class QueryParamPeriodSerializer(NoModelSerializer):
+
+    ends_at = serializers.CharField(required=False,
+        help_text=_("Data/time for the end of the period (in ISO format)"))
+
+    period_type = EnumField(choices=Plan.INTERVAL_CHOICES, required=False,
+        help_text=_("Natural period length"\
+        " (hourly, daily, weekly, monthly, yearly). Defaults to monthly."))
+
+    nb_periods = serializers.IntegerField(
+        required=False,
+        min_value=1,
+        max_value=100,
+        help_text=_("Specify the number of periods to include. "
+        "Min value is 1."))
+
+
+class QueryParamRoleStatusSerializer(NoModelSerializer):
+
+    role_status = serializers.CharField(required=False, default='',
+        allow_blank=True)
+
+
+class QueryParamPersonalProfSerializer(NoModelSerializer):
+
+    include_personal_profile = serializers.BooleanField(required=False,
+            help_text=_("True when a personal profile should be shown"),
+            default=None, allow_null=True)
+
+    convert_from_personal = serializers.BooleanField(required=False,
+            default=None, allow_null=True)
+
+
+class QueryParamUpdateSerializer(NoModelSerializer):
+
+    update = serializers.BooleanField(required=False,
+        help_text=_("Adds context to update a payment method on file."))
