@@ -29,12 +29,12 @@ import logging
 from django.http import Http404
 from rest_framework import status
 from rest_framework.generics import (get_object_or_404, GenericAPIView,
-    ListAPIView, ListCreateAPIView, RetrieveUpdateDestroyAPIView)
+    ListAPIView, RetrieveUpdateDestroyAPIView)
 from rest_framework.response import Response
 
 from ..compat import gettext_lazy as _
 from ..decorators import _valid_manager
-from ..docs import OpenAPIResponse, no_body, swagger_auto_schema
+from ..docs import OpenApiResponse, extend_schema
 from ..filters import (ActiveInPeriodFilter, ChurnedInPeriodFilter,
     IntersectPeriodFilter)
 from ..mixins import (PlanProvidedSubscriptionsMixin,
@@ -43,8 +43,8 @@ from ..mixins import (PlanProvidedSubscriptionsMixin,
 from .. import settings, signals
 from ..models import Subscription
 from ..utils import generate_random_slug, datetime_or_now
-from .roles import OptinBase
-from .serializers import (ForceSerializer,
+from .roles import ListOptinAPIView
+from .serializers import (QueryParamForceSerializer,
     ProvidedSubscriptionSerializer, ProvidedSubscriptionCreateSerializer,
     ProvidedSubscriptionDetailSerializer,SubscribedSubscriptionSerializer)
 
@@ -391,7 +391,6 @@ class PlanSubscriptionDetailAPIView(SubscriptionDetailMixin,
           },
           "description": null,
           "auto_renew": true,
-          "editable": true,
           "extra": null,
           "grant_key": null,
           "request_key": null
@@ -468,7 +467,6 @@ class PlanSubscriptionDetailAPIView(SubscriptionDetailMixin,
                 },
                 "description": null,
                 "auto_renew": true,
-                "editable": true,
                 "extra": null,
                 "grant_key": null,
                 "request_key": null
@@ -527,8 +525,7 @@ class PlanAllSubscribersAPIView(SubscriptionSmartListMixin,
 
 
 class PlanActiveSubscribersBaseAPIView(PlanProvidedSubscriptionsMixin,
-                                       OptinBase, ListCreateAPIView):
-
+                                       ListOptinAPIView):
     pass
 
 
@@ -634,9 +631,8 @@ class PlanActiveSubscribersAPIView(SubscriptionSmartListMixin,
             subscriptions += [subscription]
         return subscriptions, created
 
-    @swagger_auto_schema(responses={
-      201: OpenAPIResponse("created", ProvidedSubscriptionSerializer)},
-        query_serializer=ForceSerializer)
+    @extend_schema(parameters=[QueryParamForceSerializer], responses={
+      201: OpenApiResponse(ProvidedSubscriptionSerializer)})
     def post(self, request, *args, **kwargs):
         """
         Grants a subscription
@@ -1011,11 +1007,16 @@ class SubscriptionRequestAcceptAPIView(GenericAPIView):
         return self._subscription
 
     def get_queryset(self):
-        return Subscription.objects.active_with(
+        queryset = Subscription.objects.active_with(
             self.kwargs.get(self.provider_url_kwarg))
+        # `ProvidedSubscriptionSerializer` derives from `SubscriptionSerializer`
+        # thus will expand `organization` and `plan`.
+        queryset = queryset.select_related('organization').select_related(
+            'plan')
+        return queryset
 
-    @swagger_auto_schema(request_body=no_body, responses={
-      200: OpenAPIResponse("Grant successful", ProvidedSubscriptionSerializer)})
+    @extend_schema(request=None, responses={
+      200: OpenApiResponse(ProvidedSubscriptionSerializer)})
     def post(self, request, *args, **kwargs):
         """
         Grants a subscription request
@@ -1053,7 +1054,6 @@ a00000d0a0000001234567890123456789012345 HTTP/1.1
                 "title": "Open Space"
               },
               "auto_renew": true,
-              "editable": true,
               "extra": null,
               "grant_key": null,
               "request_key": null
