@@ -1,4 +1,4 @@
-# Copyright (c) 2022, DjaoDjin inc.
+# Copyright (c) 2023, DjaoDjin inc.
 # All rights reserved.
 #
 # Redistribution and use in source and binary forms, with or without
@@ -26,25 +26,24 @@ from __future__ import unicode_literals
 from collections import OrderedDict
 
 from django.db.models import Q
-from rest_framework import status
+from rest_framework import generics, status, response as http
 from rest_framework.exceptions import PermissionDenied, ValidationError
-from rest_framework.generics import CreateAPIView, GenericAPIView, ListAPIView
 from rest_framework.pagination import PageNumberPagination
-from rest_framework.response import Response
 
-from .serializers import (CreateOfflineTransactionSerializer,
-    OfflineTransactionSerializer, OrganizationBalanceSerializer,
+from .serializers import (CartItemCreateSerializer,
+    CreateOfflineTransactionSerializer, OfflineTransactionSerializer,
     TransactionSerializer)
 from ..compat import gettext_lazy as _
 from ..decorators import _valid_manager
-from ..docs import swagger_auto_schema, OpenAPIResponse
+from ..docs import extend_schema, OpenApiResponse
 from ..filters import DateRangeFilter, OrderingFilter, SearchFilter
 from ..mixins import OrganizationMixin, ProviderMixin, DateRangeContextMixin
-from ..models import get_broker, sum_orig_amount, Subscription, Transaction, Plan
+from ..models import (get_broker, record_use_charge, sum_orig_amount,
+    Subscription, Transaction, Plan)
 from ..backends import ProcessorError
 from ..pagination import (BalancePagination, StatementBalancePagination,
     TotalPagination)
-from ..utils import get_organization_model
+from ..utils import datetime_or_now, get_organization_model
 
 
 class IncludesSyncErrorPagination(PageNumberPagination):
@@ -65,27 +64,45 @@ class IncludesSyncErrorPagination(PageNumberPagination):
         ]
         if hasattr(self, 'detail'):
             paginated += [('detail', self.detail)]
-        return Response(OrderedDict(paginated))
+        return http.Response(OrderedDict(paginated))
 
 
 class TotalAnnotateMixin(object):
 
     def get_queryset(self):
         queryset = super(TotalAnnotateMixin, self).get_queryset()
-        balances = sum_orig_amount(queryset)
-        if len(balances) > 1:
-            raise ValueError(_("balances with multiple currency units (%s)") %
-                str(balances))
-        # `sum_orig_amount` guarentees at least one result.
-        self.totals = balances[0]
+        self.totals = sum_orig_amount(queryset)
         return queryset
 
 
-class TransactionFilterMixin(DateRangeContextMixin):
-    """
-    ``Transaction`` list result of a search query, filtered by dates.
+class SmartTransactionListMixin(DateRangeContextMixin):
     """
+    The queryset can be further filtered to a range of dates between
+    ``start_at`` and ``ends_at``.
 
+    The queryset can be further filtered by passing a ``q`` parameter.
+    The value in ``q`` will be matched against:
+
+      - description
+      - dest_profile
+      - dest_profile__full_name
+      - orig_profile
+      - orig_profile__full_name
+
+    The result queryset can be ordered by passing an ``o`` (field name)
+    and ``ot`` (asc or desc) parameter.
+    The fields the queryset can be ordered by are:
+
+      - descr
+      - dest_amount
+      - dest_organization__slug
+      - dest_organization__full_name
+      - dest_account
+      - orig_organization__slug
+      - orig_organization__full_name
+      - orig_account
+      - created_at
+    """
     search_fields = (
         'description',
         'dest_profile',
@@ -100,14 +117,6 @@ class TransactionFilterMixin(DateRangeContextMixin):
         'orig_profile': 'orig_organization__slug',
         'orig_profile__full_name': 'orig_organization__full_name',
     }
-
-    filter_backends = (DateRangeFilter, SearchFilter)
-
-
-class SmartTransactionListMixin(TransactionFilterMixin):
-    """
-    ``Transaction`` list which is also searchable and sortable.
-    """
     ordering_fields = (
         ('descr', 'description'),
         ('dest_amount', 'amount'),
@@ -121,8 +130,8 @@ class SmartTransactionListMixin(TransactionFilterMixin):
     )
     ordering = ('created_at',)
 
-    filter_backends = (TransactionFilterMixin.filter_backends +
-        (OrderingFilter,))
+
+    filter_backends = (DateRangeFilter, SearchFilter, OrderingFilter)
 
 
 class TransactionQuerysetMixin(object):
@@ -130,14 +139,21 @@ class TransactionQuerysetMixin(object):
     def get_queryset(self):
         self.selector = self.request.GET.get('selector', None)
         if self.selector is not None:
-            return Transaction.objects.filter(
+            queryset = Transaction.objects.filter(
                 Q(dest_account__icontains=self.selector)
                 | Q(orig_account__icontains=self.selector))
-        return Transaction.objects.all()
+        else:
+            queryset = Transaction.objects.all()
+        # `TransactionSerializer` will expand `orig_organization`
+        # and `dest_organization` so we add `select_related` for the ORM
+        # to generate expected SQL.
+        queryset = queryset.select_related(
+            'orig_organization').select_related('dest_organization')
+        return queryset
 
 
 class TransactionListAPIView(SmartTransactionListMixin,
-                             TransactionQuerysetMixin, ListAPIView):
+                             TransactionQuerysetMixin, generics.ListAPIView):
     """
     Lists ledger transactions
 
@@ -212,7 +228,7 @@ class BillingsQuerysetMixin(OrganizationMixin):
 
 
 class BillingsAPIView(SmartTransactionListMixin,
-                      BillingsQuerysetMixin, ListAPIView):
+                      BillingsQuerysetMixin, generics.ListAPIView):
     """
     Lists subscriber transactions
 
@@ -289,11 +305,17 @@ class ReceivablesQuerysetMixin(ProviderMixin):
         """
         Get the list of transactions for this organization.
         """
-        return self.provider.receivables().filter(orig_amount__gt=0)
+        queryset = self.provider.receivables().filter(orig_amount__gt=0)
+        # `TransactionSerializer` will expand `orig_organization`
+        # and `dest_organization` so we add `select_related` for the ORM
+        # to generate expected SQL.
+        queryset = queryset.select_related(
+            'orig_organization').select_related('dest_organization')
+        return queryset
 
 
 class ReceivablesListAPIView(TotalAnnotateMixin, SmartTransactionListMixin,
-                             ReceivablesQuerysetMixin, ListAPIView):
+                             ReceivablesQuerysetMixin, generics.ListAPIView):
     """
     Lists provider receivables
 
@@ -378,7 +400,7 @@ class TransferQuerysetMixin(ProviderMixin):
 
 
 class TransferListAPIView(SmartTransactionListMixin, TransferQuerysetMixin,
-                          ListAPIView):
+                          generics.ListAPIView):
     """
     Lists provider payouts
 
@@ -445,12 +467,12 @@ class TransferListAPIView(SmartTransactionListMixin, TransferQuerysetMixin,
     pagination_class = IncludesSyncErrorPagination
 
 
-class ImportTransactionsAPIView(ProviderMixin, CreateAPIView):
+class ImportTransactionsAPIView(ProviderMixin, generics.CreateAPIView):
 
     serializer_class = CreateOfflineTransactionSerializer
 
-    @swagger_auto_schema(responses={
-        201: OpenAPIResponse("", OfflineTransactionSerializer)})
+    @extend_schema(responses={
+        201: OpenApiResponse(OfflineTransactionSerializer)})
     def post(self, request, *args, **kwargs):
         """
         Creates an offline transaction
@@ -655,11 +677,18 @@ class ImportTransactionsAPIView(ProviderMixin, CreateAPIView):
                 many=True).to_representation(transactions)
         }
         headers = self.get_success_headers(result_data)
-        return Response(result_data,
+        return http.Response(result_data,
             status=status.HTTP_201_CREATED, headers=headers)
 
 
-class StatementBalanceAPIView(OrganizationMixin, GenericAPIView):
+class StatementBalanceQuerysetMixin(OrganizationMixin):
+
+    def get_queryset(self):
+        return self.organization.last_unpaid_orders()
+
+
+class StatementBalanceAPIView(SmartTransactionListMixin,
+                    StatementBalanceQuerysetMixin, generics.ListCreateAPIView):
     """
     Retrieves a customer balance
 
@@ -679,19 +708,131 @@ class StatementBalanceAPIView(OrganizationMixin, GenericAPIView):
 
         {
             "balance_amount": "1200",
-            "balance_unit": "usd"
+            "balance_unit": "usd",
+            "start_at": "2023-01-01T00:00:00Z",
+            "ends_at": "2023-06-01T23:42:13.863739Z",
+            "count": 1,
+            "next": null,
+            "previous": null,
+            "results": [
+                {
+                  "created_at":"2016-06-21T23:42:13.863739Z",
+                  "description":"Subscription to basic until 2016/11/21 (1 month)",
+                  "amount":"$20.00",
+                  "is_debit":false,
+                  "orig_account":"Receivable",
+                  "orig_profile": {
+                      "slug": "cowork",
+                      "printable_name": "Coworking Space",
+                      "picture": null,
+                      "type": "organization",
+                      "credentials": false
+                  },
+                  "orig_amount":2000,
+                  "orig_unit":"usd",
+                  "dest_account":"Payable",
+                  "dest_profile": {
+                      "slug": "xia",
+                      "printable_name": "Xia Lee",
+                      "picture": null,
+                      "type": "personal",
+                      "credentials": true
+                  },
+                  "dest_amount":2000,
+                  "dest_unit":"usd"
+                }]
         }
     """
-    serializer_class = OrganizationBalanceSerializer
-    pagination_class = None
+    pagination_class = BalancePagination
+    serializer_class = TransactionSerializer
+
+    ordering = ('-created_at',)
+
+    def get_serializer_class(self):
+        if self.request.method.lower() in ('post',):
+            return CartItemCreateSerializer
+        return super(StatementBalanceAPIView, self).get_serializer_class()
 
     def get(self, request, *args, **kwargs):
-        #pylint:disable=unused-argument
-        balance_amount, balance_unit \
+        #pylint:disable=attribute-defined-outside-init
+        self.balance_amount, self.balance_unit \
             = Transaction.objects.get_statement_balance(self.organization)
-        return Response(self.get_serializer_class()().to_representation(
-            {'balance_amount': balance_amount,
-             'balance_unit': balance_unit}))
+        return super(StatementBalanceAPIView, self).get(
+            request, *args, **kwargs)
+
+    @extend_schema(responses={
+        201: OpenApiResponse(TransactionSerializer)})
+    def post(self, request, *args, **kwargs):
+        """
+        Adds to the order balance
+
+        This API endpoint can be used to add use charges to a subscriber
+        invoice while charging the subscriber at a later date.
+
+        **Tags**: billing, subscriber
+
+        **Examples**
+
+        .. code-block:: http
+
+            POST /api/billing/xia/balance HTTP/1.1
+
+        .. code-block:: json
+
+            {
+                "plan": "basic",
+                "use": "requests"
+            }
+
+        responds
+
+        .. code-block:: json
+
+            {
+              "created_at":"2016-06-21T23:42:13.863739Z",
+              "description":"Subscription to basic until 2016/11/21 (1 month)",
+              "amount":"$20.00",
+              "is_debit":false,
+              "orig_account":"Receivable",
+              "orig_profile": {
+                  "slug": "cowork",
+                  "printable_name": "Coworking Space",
+                  "picture": null,
+                  "type": "organization",
+                  "credentials": false
+              },
+              "orig_amount":2000,
+              "orig_unit":"usd",
+              "dest_account":"Payable",
+              "dest_profile": {
+                  "slug": "xia",
+                  "printable_name": "Xia Lee",
+                  "picture": null,
+                  "type": "personal",
+                  "credentials": true
+              },
+              "dest_amount":2000,
+              "dest_unit":"usd"
+            }
+        """
+        return self.create(request, *args, **kwargs)
+
+    def create(self, request, *args, **kwargs):
+        serializer = self.get_serializer(data=request.data)
+        serializer.is_valid(raise_exception=True)
+        plan = serializer.validated_data.get('plan')
+        use_charge = serializer.validated_data.get('use')
+        quantity = serializer.validated_data.get('quantity')
+        if use_charge:
+            subscription = Subscription.objects.valid_for(
+                organization=self.organization, plan=plan,
+                ends_at__gt=datetime_or_now()).first()
+            order_executed_items = record_use_charge(
+                subscription, use_charge, quantity=quantity)
+            serializer = self.serializer_class(instance=order_executed_items[0])
+            return http.Response(serializer.data, status=status.HTTP_201_CREATED)
+        return http.Response({}, status=status.HTTP_200_OK)
+
 
     def delete(self, request, *args, **kwargs):
         """
@@ -719,4 +860,4 @@ class StatementBalanceAPIView(OrganizationMixin, GenericAPIView):
             # to subscribers and providers.
             raise PermissionDenied()
         self.organization.create_cancel_transactions(user=request.user)
-        return Response(status=status.HTTP_204_NO_CONTENT)
+        return http.Response(status=status.HTTP_204_NO_CONTENT)
