@@ -1,4 +1,4 @@
-# Copyright (c) 2022, DjaoDjin inc.
+# Copyright (c) 2023, DjaoDjin inc.
 # All rights reserved.
 #
 # Redistribution and use in source and binary forms, with or without
@@ -25,15 +25,13 @@ from __future__ import unicode_literals
 
 from collections import OrderedDict
 
-from django.http import Http404
 from django.shortcuts import get_object_or_404
 
 # Implementation Note:
 # pylint:disable=import-outside-toplevel
 # saas.settings cannot be imported at this point because this file (extras.py)
 # will be imported before ``django.conf.settings`` is fully initialized.
-from .compat import (NoReverseMatch, gettext_lazy as _, is_authenticated,
-    reverse, six)
+from .compat import NoReverseMatch, is_authenticated, reverse, six
 from .utils import get_organization_model
 
 
@@ -42,8 +40,7 @@ class OrganizationMixinBase(object):
     Returns an ``Organization`` from a URL.
     """
 
-    organization_url_kwarg = 'organization'  # XXX PROFILE_URL_KWARG
-    #organization_url_kwarg = 'profile'       # XXX PROFILE_URL_KWARG
+    organization_url_kwarg = 'profile'  # Also modify settings.PROFILE_URL_KWARG
 
     @property
     def organization(self):
@@ -68,7 +65,6 @@ class OrganizationMixinBase(object):
 
     def get_context_data(self, **kwargs):
         context = super(OrganizationMixinBase, self).get_context_data(**kwargs)
-
         organization = self.organization
         if not organization:
             return context
@@ -84,6 +80,7 @@ class OrganizationMixinBase(object):
         # URLs for both sides (subscriber and provider).
         urls.update({
             'profile_base': reverse('saas_profile'),
+            'user_profiles': reverse('saas_api_user_profiles', args=(organization,)),
             'organization': {
                 'api_base': reverse(
                     'saas_api_organization', args=(organization,)),
@@ -161,6 +158,8 @@ class OrganizationMixinBase(object):
                     'saas_metrics_lifetimevalue', args=(provider,)),
                 'ledger_balances': reverse(
                     'saas_balance', args=(provider,)),
+                'metrics_balances_due': reverse(
+                    'saas_metrics_balances_due', args=(provider,)),
                 'profile': reverse('saas_provider_profile'),
                 'subscribers': reverse(
                     'saas_subscriber_list', args=(provider,)),
