@@ -28,7 +28,6 @@ from functools import reduce
 
 from django.core.exceptions import FieldDoesNotExist
 from django.db import models
-from django.utils.timezone import utc
 from rest_framework.filters import (OrderingFilter as BaseOrderingFilter,
     SearchFilter as BaseSearchFilter, BaseFilterBackend)
 from rest_framework.compat import distinct
@@ -40,10 +39,68 @@ from .utils import datetime_or_now, parse_tz
 LOGGER = logging.getLogger(__name__)
 
 
+def search_terms_as_list(params):
+    """
+    Search terms are set by a ?q=... query parameter.
+    When multiple search terms must be matched, they can be delimited
+    by a comma.
+    """
+    params = params.replace('\x00', '')  # strip null characters
+    results = []
+    inside = False
+    first = 0
+    for last, letter in enumerate(params):
+        if inside:
+            if letter == '"':
+                if first < last:
+                    results += [params[first:last]]
+                first = last + 1
+                inside = False
+        else:
+            if letter in (',',):
+                if first < last:
+                    results += [params[first:last]]
+                first = last + 1
+            elif letter == '"':
+                inside = True
+                first = last + 1
+    if first < len(params):
+        results += [params[first:len(params)]]
+    return results
+
+
+class ActiveFilter(BaseSearchFilter):
+    """
+    All items which have `is_active == True` only.
+    """
+    is_active_param = 'active'
+
+    def filter_queryset(self, request, queryset, view):
+        is_active = request.query_params.get(self.is_active_param)
+        if is_active is not None:
+            queryset = queryset.filter(is_active=bool(is_active))
+        return queryset
+
+
+    def get_schema_operation_parameters(self, view):
+        fields = super(ActiveFilter, self).get_schema_operation_parameters(view)
+        fields += [{
+            'name': self.is_active_param,
+            'required': False,
+            'in': 'query',
+            'description': force_str("True when customers can subscribe"\
+                " to the plan"),
+            'schema': {
+                'type': 'string',
+            },
+        }]
+        return fields
+
+
 class SearchFilter(BaseSearchFilter):
 
     search_field_param = settings.SEARCH_FIELDS_PARAM
-
+    mail_provider_domains = settings.MAIL_PROVIDER_DOMAINS
 
     def filter_queryset(self, request, queryset, view):
         search_fields = self.get_valid_fields(request, queryset, view)
@@ -67,6 +124,12 @@ class SearchFilter(BaseSearchFilter):
                 for orm_lookup in orm_lookups
             ]
             conditions.append(reduce(operator.or_, queries))
+            if ('@' in search_term and 'domain' in view.search_fields and
+                'email' in search_fields):
+                domain = '@' + search_term.split('@')[-1]
+                if domain not in self.mail_provider_domains:
+                    queries = [models.Q(**{'email__iendswith': domain})]
+                    conditions.append(reduce(operator.or_, queries))
         queryset = queryset.filter(reduce(operator.or_, conditions))
 
         if self.must_call_distinct(queryset, search_fields):
@@ -78,8 +141,7 @@ class SearchFilter(BaseSearchFilter):
         return queryset
 
 
-    @staticmethod
-    def filter_valid_fields(queryset, fields, view):
+    def filter_valid_fields(self, queryset, fields, view):
         #pylint:disable=protected-access
         model_fields = {
             field.name for field in queryset.model._meta.get_fields()}
@@ -87,8 +149,14 @@ class SearchFilter(BaseSearchFilter):
         # which are not present in the model.
         alternate_fields = getattr(view, 'alternate_fields', {})
         for field in fields:
+            field_lookup = field[0]
+            lookup = self.lookup_prefixes.get(field_lookup)
+            if lookup:
+                field = field[1:]
             alternate_field = alternate_fields.get(field, None)
             if alternate_field:
+                if lookup:
+                    alternate_field = field_lookup + alternate_field
                 if isinstance(alternate_field, (list, tuple)):
                     fields += tuple(alternate_field)
                 else:
@@ -96,8 +164,12 @@ class SearchFilter(BaseSearchFilter):
 
         valid_fields = []
         for field in fields:
-            if '__' in field:
-                relation, rel_field = field.split('__')
+            field_name = field
+            lookup = self.lookup_prefixes.get(field_name[0])
+            if lookup:
+                field_name = field_name[1:]
+            if '__' in field_name:
+                relation, rel_field = field_name.split('__')
                 try:
                     # check if the field is a relation
                     rel = queryset.model._meta.get_field(relation).remote_field
@@ -108,8 +180,8 @@ class SearchFilter(BaseSearchFilter):
                         valid_fields.append(field)
                 except FieldDoesNotExist:
                     pass
-            elif field in model_fields:
-                rel = queryset.model._meta.get_field(field).remote_field
+            elif field_name in model_fields:
+                rel = queryset.model._meta.get_field(field_name).remote_field
                 if not rel:
                     # if it is a relation fields (as a result valid),
                     # we don't want to end-up with a problem later on
@@ -119,8 +191,24 @@ class SearchFilter(BaseSearchFilter):
         return tuple(valid_fields)
 
 
+    def get_query_param(self, request, key, default_value=None):
+        try:
+            return request.query_params.get(key, default_value)
+        except AttributeError:
+            pass
+        return request.GET.get(key, default_value)
+
+
+    def get_query_param_list(self, request, key, default_value=None):
+        try:
+            return request.query_params.getlist(key, default_value)
+        except AttributeError:
+            pass
+        return request.GET.getlist(key, default_value)
+
+
     def get_query_fields(self, request):
-        return request.query_params.getlist(self.search_field_param)
+        return self.get_query_param_list(request, self.search_field_param)
 
 
     def get_search_terms(self, request):
@@ -129,29 +217,8 @@ class SearchFilter(BaseSearchFilter):
         When multiple search terms must be matched, they can be delimited
         by a comma.
         """
-        params = request.query_params.get(self.search_param, '')
-        params = params.replace('\x00', '')  # strip null characters
-        results = []
-        inside = False
-        first = 0
-        for last, letter in enumerate(params):
-            if inside:
-                if letter == '"':
-                    if first < last:
-                        results += [params[first:last]]
-                    first = last + 1
-                    inside = False
-            else:
-                if letter in (',',):
-                    if first < last:
-                        results += [params[first:last]]
-                    first = last + 1
-                elif letter == '"':
-                    inside = True
-                    first = last + 1
-        if first < len(params):
-            results += [params[first:len(params)]]
-        return results
+        return search_terms_as_list(
+            self.get_query_param(request, self.search_param, ''))
 
 
     def get_valid_fields(self, request, queryset, view, context=None):
@@ -210,6 +277,13 @@ class SearchFilter(BaseSearchFilter):
 
 class OrderingFilter(BaseOrderingFilter):
 
+    def get_query_param(self, request, key, default_value=None):
+        try:
+            return request.query_params.getlist(key, default_value)
+        except AttributeError:
+            pass
+        return request.GET.get(key, default_value)
+
     def get_valid_fields(self, queryset, view, context=None):
         #pylint:disable=protected-access
         if context is None:
@@ -274,7 +348,7 @@ class OrderingFilter(BaseOrderingFilter):
             ordering = self.remove_invalid_fields(
                 queryset, default, view, request)
         default_ordering = list(ordering)
-        params = request.query_params.getlist(self.ordering_param)
+        params = self.get_query_param(request, self.ordering_param)
         if params:
             if isinstance(params, six.string_types):
                 params = params.split(',')
@@ -329,27 +403,30 @@ class DateRangeFilter(BaseFilterBackend):
     forced_date_range = True
     date_field = 'created_at'
     alternate_date_field = 'date_joined'
-    start_at_param = 'start_at'
     ends_at_param = 'ends_at'
+    start_at_param = 'start_at'
+    timezone_param = None # 'timezone'
 
     def get_params(self, request, view):
-        tz_ob = parse_tz(request.GET.get('timezone'))
+        tz_ob = None
+        if self.timezone_param:
+            tz_ob = parse_tz(request.query_params.get(self.timezone_param))
         if not tz_ob:
-            tz_ob = utc
+            organization = getattr(view, 'organization', None)
+            if organization:
+                tz_ob = parse_tz(organization.default_timezone)
         ends_at = None
         if self.ends_at_param:
-            ends_at = request.GET.get(self.ends_at_param)
-        start_at = request.GET.get(self.start_at_param)
+            ends_at = request.query_params.get(self.ends_at_param)
+        start_at = request.query_params.get(self.start_at_param)
         forced_date_range = getattr(view, 'forced_date_range',
             self.forced_date_range)
         if forced_date_range or ends_at:
             if ends_at is not None:
                 ends_at = ends_at.strip('"')
-            ends_at = datetime_or_now(ends_at)
-            ends_at = ends_at.astimezone(tz_ob)
+            ends_at = datetime_or_now(ends_at, tzinfo=tz_ob)
         if start_at:
-            start_at = datetime_or_now(start_at.strip('"'))
-            start_at = start_at.astimezone(tz_ob)
+            start_at = datetime_or_now(start_at.strip('"'), tzinfo=tz_ob)
         return start_at, ends_at
 
     def get_date_field(self, model):
@@ -377,21 +454,32 @@ class DateRangeFilter(BaseFilterBackend):
     def get_schema_operation_parameters(self, view):
         fields = super(DateRangeFilter, self).get_schema_operation_parameters(
             view)
-        fields += [{
-            'name': self.start_at_param,
-            'required': False,
-            'in': 'query',
-            'description': force_str("date/time in ISO format"),
-            'schema': {
-                'type': 'string',
-            },
-        }]
+        if self.start_at_param:
+            fields += [{
+                'name': self.start_at_param,
+                'required': False,
+                'in': 'query',
+                'description': force_str("date/time in ISO 8601 format"),
+                'schema': {
+                    'type': 'string',
+                },
+            }]
         if self.ends_at_param:
             fields += [{
                 'name': self.ends_at_param,
                 'required': False,
                 'in': 'query',
-                'description': force_str("date/time in ISO format"),
+                'description': force_str("date/time in ISO 8601 format"),
+                'schema': {
+                    'type': 'string',
+                },
+            }]
+        if self.timezone_param:
+            fields += [{
+                'name': self.timezone_param,
+                'required': False,
+                'in': 'query',
+                'description': force_str("timezone"),
                 'schema': {
                     'type': 'string',
                 },
