@@ -32,6 +32,7 @@ from django.contrib.auth.hashers import UNUSABLE_PASSWORD_PREFIX
 from django.db import models, transaction, IntegrityError
 from django.http import Http404
 from django.template.defaultfilters import slugify
+from django.template.loader import render_to_string
 from rest_framework.generics import get_object_or_404
 
 from . import humanize, settings
@@ -43,8 +44,9 @@ from .models import (CartItem, Charge, Coupon, Plan, Price,
     RoleDescription, Subscription, Transaction, get_broker, sum_orig_amount)
 from .utils import (build_absolute_uri, datetime_or_now,
     full_name_natural_split, get_organization_model, get_role_model,
-    handle_uniq_error, update_context_urls, validate_redirect_url)
+    handle_uniq_error, parse_tz, update_context_urls, validate_redirect_url)
 from .extras import OrganizationMixinBase
+from .metrics.transactions import get_balances_due
 
 LOGGER = logging.getLogger(__name__)
 
@@ -403,7 +405,7 @@ class UserMixin(object):
         if self.user:
             top_accessibles = []
             queryset = get_organization_model().objects.accessible_by(
-                self.user).filter(is_active=True).exclude(
+                self.user).exclude(
                 slug=self.user.username)[:self.SHORT_LIST_CUT_OFF + 1]
             for organization in queryset:
                 if organization.is_provider:
@@ -559,7 +561,7 @@ class OrganizationDecorateMixin(object):
         # A raw query cannot be furthered filtered either.
         organization_model = get_organization_model()
         records = [page] if isinstance(page, organization_model) else page
-        personal = dict(organization_model.objects.filter(
+        personal = dict(organization_model.objects.filter(#XXX ok if not active?
             pk__in=[profile.pk for profile in records if profile.pk],
             role__user__username=models.F('slug')).extra(select={
             'credentials': ("NOT (password LIKE '" + UNUSABLE_PASSWORD_PREFIX
@@ -596,38 +598,57 @@ class ChargeMixin(OrganizationMixin):
 
 class DateRangeContextMixin(object):
 
+    ends_at_param = 'ends_at'
+    start_at_param = 'start_at'
+    timezone_param = 'timezone'
     forced_date_range = True
 
+    def get_query_param(self, key, default_value=None):
+        try:
+            return self.request.query_params.get(key, default_value)
+        except AttributeError:
+            pass
+        return self.request.GET.get(key, default_value)
+
     @property
     def start_at(self):
         if not hasattr(self, '_start_at'):
-            self._start_at = self.request.GET.get('start_at', None)
+            self._start_at = self.get_query_param(self.start_at_param, None)
             if self._start_at:
-                self._start_at = datetime_or_now(self._start_at.strip('"'))
+                self._start_at = datetime_or_now(self._start_at.strip('"'),
+                    tzinfo=self.timezone)
         return self._start_at
 
     @property
     def ends_at(self):
         if not hasattr(self, '_ends_at'):
-            self._ends_at = self.request.GET.get('ends_at', None)
+            self._ends_at = self.get_query_param(self.ends_at_param, None)
             if self.forced_date_range or self._ends_at:
                 if self._ends_at is not None:
                     self._ends_at = self._ends_at.strip('"')
-                self._ends_at = datetime_or_now(self._ends_at)
+                self._ends_at = datetime_or_now(self._ends_at,
+                    tzinfo=self.timezone)
         return self._ends_at
 
     @property
     def timezone(self):
         if not hasattr(self, '_timezone'):
-            self._timezone = self.request.GET.get('timezone', None)
+            self._timezone = parse_tz(self.get_query_param(
+                self.timezone_param, None))
+            if not self._timezone:
+                organization = getattr(self, 'organization', None)
+                if organization:
+                    self._timezone = parse_tz(organization.default_timezone)
         return self._timezone
 
     def get_context_data(self, **kwargs):
         context = super(DateRangeContextMixin, self).get_context_data(**kwargs)
         if self.start_at:
-            context.update({'start_at': self.start_at})
+            context.update({self.start_at_param: self.start_at})
         if self.ends_at:
-            context.update({'ends_at': self.ends_at})
+            context.update({self.ends_at_param: self.ends_at})
+        if self.timezone:
+            context.update({self.timezone_param: self.timezone})
         return context
 
 
@@ -659,10 +680,12 @@ class InvoicablesMixin(OrganizationMixin):
     @property
     def invoicables_provider(self):
         if not hasattr(self, '_invoicables_provider'):
+            organization_model = get_organization_model()
             providers = set([])
             for invoicable in self.invoicables:
-                providers |= set(Transaction.objects.providers(
-                    invoicable['lines']))
+                providers |= set(
+                    organization_model.objects.receivable_providers(
+                        invoicable['lines']))
             if len(providers) == 1:
                 self._invoicables_provider = list(providers)[0]
             else:
@@ -804,12 +827,7 @@ class CouponMixin(ProviderMixin):
         return context
 
 
-class MetricsMixin(DateRangeContextMixin, ProviderMixin):
-
-    filter_backends = (DateRangeFilter,)
-
-
-class CartItemSmartListMixin(object):
+class CartItemSmartListMixin(DateRangeContextMixin):
     """
     The queryset can be further filtered to a range of dates between
     ``start_at`` and ``ends_at``.
@@ -843,36 +861,15 @@ class CartItemSmartListMixin(object):
     )
     ordering = ('created_at',)
 
-    filter_backends = (DateRangeFilter, OrderingFilter, SearchFilter)
+    filter_backends = (DateRangeFilter, SearchFilter, OrderingFilter)
+        # XXX `CartItemDownloadView` downloads all coupon uses (ProviderMixin),
+        # while `CouponUsesAPIView` shows only recorded use.
 
 
-class OrganizationSmartListMixin(object):
-    """
-    The queryset can be further filtered to a range of dates between
-    ``start_at`` and ``ends_at``.
-
-    The queryset can be further filtered by passing a ``q`` parameter.
-    The value in ``q`` will be matched against:
-
-      - slug
-      - full_name
-      - email
-      - phone
-      - street_address
-      - locality
-      - region
-      - postal_code
-      - country
-
-    The result queryset can be ordered by passing an ``o`` (field name)
-    and ``ot`` (asc or desc) parameter.
-    The fields the queryset can be ordered by are:
-
-      - full_name
-      - created_at
-    """
-    forced_date_range = False
-
+class OrganizationSearchOrderListMixin(object):
+    # This class does not include a `DateRangeFilter` such that we get
+    # all unengaged profiles in `UnengagedSubscribersAPIView`, not just
+    # the unengaged organizations that were created within the date range.
     alternate_fields = {
         'slug': 'username',
         'full_name': ('first_name', 'last_name'),
@@ -906,7 +903,39 @@ class OrganizationSmartListMixin(object):
     #            queryset, self.get_default_ordering(view), view, request)```
     ordering = ('full_name', 'first_name', 'last_name')
 
-    filter_backends = (DateRangeFilter, SearchFilter, OrderingFilter)
+    filter_backends = (SearchFilter, OrderingFilter,) # See comment above
+
+
+class OrganizationSmartListMixin(DateRangeContextMixin,
+                                 OrganizationSearchOrderListMixin):
+    """
+    The queryset can be further filtered to a range of dates between
+    ``start_at`` and ``ends_at``.
+
+    The queryset can be further filtered by passing a ``q`` parameter.
+    The value in ``q`` will be matched against:
+
+      - slug
+      - full_name
+      - email
+      - phone
+      - street_address
+      - locality
+      - region
+      - postal_code
+      - country
+
+    The result queryset can be ordered by passing an ``o`` (field name)
+    and ``ot`` (asc or desc) parameter.
+    The fields the queryset can be ordered by are:
+
+      - full_name
+      - created_at
+    """
+    forced_date_range = False
+
+    filter_backends = (
+        (DateRangeFilter,) + OrganizationSearchOrderListMixin.filter_backends)
 
 
 class RoleSmartListMixin(object):
@@ -969,7 +998,7 @@ class RoleSmartListMixin(object):
     filter_backends = (SearchFilter, OrderingFilter)
 
 
-class UserSmartListMixin(object):
+class UserSmartListMixin(DateRangeContextMixin):
     """
     ``User`` list which is also searchable and sortable.
 
@@ -989,9 +1018,20 @@ class UserSmartListMixin(object):
       - User.email
       - User.created_at
     """
-    search_fields = ('first_name',
-                     'last_name',
-                     'email')
+    alternate_fields = {
+        'slug': 'username',
+        'full_name': ('first_name', 'last_name'),
+        'created_at': 'date_joined',
+    }
+    search_fields = (
+        'username',
+        'first_name',
+        'last_name',
+        'email',
+        # fields to match Organization searches:
+        'slug',
+        'full_name',
+    )
     ordering_fields = (
         ('first_name', 'first_name'),
         ('last_name', 'last_name'),
@@ -1000,7 +1040,10 @@ class UserSmartListMixin(object):
     )
     ordering = ('first_name',)
 
-    filter_backends = (DateRangeFilter, OrderingFilter, SearchFilter)
+    filter_backends = (DateRangeFilter, SearchFilter, OrderingFilter)
+        # Classes deriving from `UserSmartListMixin` do not technically need
+        # a `ProviderMixin` (i.e. always is broker) but could benefit from it
+        # to get a consistent default timezone.
 
 
 class SubscribedSubscriptionsMixin(OrganizationDecorateMixin):
@@ -1011,8 +1054,12 @@ class SubscribedSubscriptionsMixin(OrganizationDecorateMixin):
     def get_queryset(self):
         # Use ``filter`` instead of active_for here because we want to list
         # through the API subscriptions which are pending opt-in.
-        return Subscription.objects.filter(
+        queryset = Subscription.objects.filter(
             organization__slug=self.kwargs.get(self.subscriber_url_kwarg))
+        # `SubscriptionSerializer` will expand `organization` and `plan`.
+        queryset = queryset.select_related('organization').select_related(
+            'plan')
+        return queryset
 
     def paginate_queryset(self, queryset):
         page = super(SubscribedSubscriptionsMixin, self).paginate_queryset(
@@ -1027,8 +1074,11 @@ class ProvidedSubscriptionsMixin(OrganizationDecorateMixin, ProviderMixin):
 
     def get_queryset(self):
         # OK to use ``filter`` here since we want to list all subscriptions.
-        return Subscription.objects.filter(
-            plan__organization=self.provider)
+        queryset = Subscription.objects.filter(plan__organization=self.provider)
+        # `SubscriptionSerializer` will expand `organization` and `plan`.
+        queryset = queryset.select_related('organization').select_related(
+            'plan')
+        return queryset
 
     def paginate_queryset(self, queryset):
         page = super(ProvidedSubscriptionsMixin, self).paginate_queryset(
@@ -1040,9 +1090,10 @@ class ProvidedSubscriptionsMixin(OrganizationDecorateMixin, ProviderMixin):
 class PlanProvidedSubscriptionsMixin(PlanMixin, ProvidedSubscriptionsMixin):
 
     def get_queryset(self):
-        return super(
+        queryset = super(
             PlanProvidedSubscriptionsMixin, self).get_queryset().filter(
             plan__slug=self.kwargs.get(self.plan_url_kwarg))
+        return queryset
 
 
 class SubscriptionSmartListMixin(object):
@@ -1143,8 +1194,13 @@ class RoleMixin(RoleDescriptionMixin):
             kwargs = {}
         # OK to use filter here since we want to present all pending grants
         # and requests.
-        return get_role_model().objects.filter(
+        queryset = get_role_model().objects.filter(
             organization=self.organization, user=self.user, **kwargs)
+        # `RoleSerializer` will expand `user` and `role_description`.
+        queryset = queryset.select_related('user').select_related(
+            'role_description')
+        return queryset
+
 
     def get_object(self):
         # Since there is no lookup_field for relations, we must override
@@ -1158,6 +1214,74 @@ class RoleMixin(RoleDescriptionMixin):
                 % queryset.model._meta.object_name)
 
 
+class BalancesDueMixin(DateRangeContextMixin, ProviderMixin):
+
+    search_fields = (
+        'slug',
+        'full_name',
+    )
+
+    ordering_fields = (
+        ('slug', 'Slug'),
+        ('full_name', 'Full Name'),
+        ('created_at', 'Created At')
+    )
+    ordering = ('slug',)
+
+    filter_backends = (DateRangeFilter, SearchFilter, OrderingFilter)
+
+    @property
+    def balances_due(self):
+        if not hasattr(self, "_balances_due"):
+            self._balances_due = get_balances_due(provider=self.provider)
+        return self._balances_due
+
+    def get_queryset(self):
+        organization_model = get_organization_model()
+        if self.provider:
+            queryset = organization_model.objects.filter(
+                subscribes_to__organization=self.provider).distinct()
+        else:
+            queryset = organization_model.objects.all()
+        queryset = queryset.filter(
+            outgoing__orig_account=Transaction.PAYABLE,
+            slug__in=self.balances_due.keys()).distinct()
+        return queryset.order_by('full_name')
+
+    def decorate_queryset(self, queryset):
+        decorated_queryset = list(queryset)
+        balances_values = self.balances_due
+        for organization in decorated_queryset:
+            balance = balances_values.get(organization.slug)
+            if balance:
+                for unit_data in six.itervalues(balance):
+                    unit_data.pop('Liability', None)
+                organization.balances = balance
+            else:
+                organization.balances = None
+        return decorated_queryset
+
+
+class MetricsDownloadMixin(object):
+
+    filter_backends = []
+
+    def get_queryset(self):
+        results, _ = self.get_data()
+        consolidated_data = {}
+        for result in results:
+            for date_value in result['values']:
+                date_str = date_value[0]
+                if date_str not in consolidated_data:
+                    consolidated_data[date_str] = {
+                        'Date': date_str,
+                        **{heading: 0 for heading in self.headings if heading != 'Date'}
+                    }
+                consolidated_data[date_str][result['title']] = date_value[1]
+
+        return list(consolidated_data.values())
+
+
 def _as_html_description(transaction_descr,
                          orig_organization=None, dest_organization=None,
                          dest_account=None, request=None, force_links=False):
@@ -1258,7 +1382,7 @@ def get_charge_context(charge):
         'charge': charge,
         'refunded': charge.refunded.exists(),
         'charge_items': charge.line_items,
-        'organization': charge.customer,
+        'organization': charge.customer, # XXX Use settings.PROFILE_URL_KWARG?
         'provider': charge.broker, # XXX update templates
     }
     return context
@@ -1280,3 +1404,17 @@ def product_url(subscriber=None, plan=None, request=None):
     if plan:
         location += '%s/' % plan
     return build_absolute_uri(request, location=location)
+
+
+def read_agreement_file(slug, context=None, request=None):
+    import markdown #pylint:disable=import-outside-toplevel
+    if not context:
+        broker = get_broker()
+        context = {
+            'organization': broker,
+            'site_url': build_absolute_uri(request),
+        }
+    # We use context and not context=context in the following statement
+    # such that the code is compatible with Django 1.7 and Django 1.8
+    return markdown.markdown(
+        render_to_string('saas/agreements/%s.md' % slug, context))
