@@ -80,13 +80,22 @@ _SETTINGS = {
             getattr(settings, 'BASE_DIR', "broker")),
         'IS_INSTANCE_CALLABLE': None,
         'BUILD_ABSOLUTE_URI_CALLABLE': None,
-        'FEE_PERCENTAGE': 0
+        'FEE_PERCENTAGE': 0,
+        'MINIMUM_PLAN_AMOUNT': {
+            'HOURLY': 0,
+            'DAILY': 0,
+            'WEEKLY': 0,
+            'MONTHLY': 0,
+            'YEARLY': 0,
+        }
     },
     'BYPASS_IMPLICIT_GRANT': {},
     'BYPASS_PERMISSION_CHECK': False,
     # Do not check the auth token against the processor to set processor keys.
     # (useful while testing).
     'BYPASS_PROCESSOR_AUTH': False,
+    'CURRENCY_JSON_PATH': os.path.join(os.path.dirname(__file__),
+        'static', 'data', 'currencies.json'),
     'DEFAULT_UNIT': 'usd',
     'DISABLE_UPDATES': False,
     'EXPIRE_NOTICE_DAYS': [15],
@@ -95,9 +104,11 @@ _SETTINGS = {
     'INACTIVITY_DAYS': 90,
     'MAX_RENEWAL_ATTEMPTS': 3,
     'MAX_TYPEAHEAD_CANDIDATES': 5,
+    'MAIL_PROVIDER_DOMAINS': [],
     'ORGANIZATION_MODEL': getattr(settings, 'SAAS_ORGANIZATION_MODEL',
         'saas.Organization'),
     'PAGE_SIZE': 25,
+    'PHONE_VERIFICATION_BACKEND': None,
     'PICTURE_STORAGE_CALLABLE': None,
     'PLATFORM_NAME': getattr(settings, 'APP_NAME',
         os.path.basename(settings.BASE_DIR)),
@@ -126,8 +137,7 @@ _SETTINGS = {
     'TERMS_OF_USE': 'terms-of-use',
     'MANAGER': 'manager',
     'CONTRIBUTOR': 'contributor',
-    'PROFILE_URL_KWARG': 'organization'
-#XXX   'PROFILE_URL_KWARG': 'profile'
+    'PROFILE_URL_KWARG': 'profile' #Also modify organization_url_kwarg in extras
 }
 _SETTINGS.update(getattr(settings, 'SAAS', {}))
 
@@ -147,6 +157,16 @@ AUTH_USER_MODEL = getattr(
 #: has its own database of users, profiles, etc.
 BROKER_CALLABLE = _SETTINGS.get('BROKER').get('GET_INSTANCE', None)
 BROKER_FEE_PERCENTAGE = _SETTINGS.get('BROKER').get('FEE_PERCENTAGE', 0)
+BROKER_MINIMUM_PLAN_AMOUNT_HOURLY = _SETTINGS.get('BROKER').get(
+    'MINIMUM_PLAN_AMOUNT', {}).get('HOURLY', 0)
+BROKER_MINIMUM_PLAN_AMOUNT_DAILY = _SETTINGS.get('BROKER').get(
+    'MINIMUM_PLAN_AMOUNT', {}).get('HOURLY', 0)
+BROKER_MINIMUM_PLAN_AMOUNT_WEEKLY = _SETTINGS.get('BROKER').get(
+    'MINIMUM_PLAN_AMOUNT', {}).get('HOURLY', 0)
+BROKER_MINIMUM_PLAN_AMOUNT_MONTHLY = _SETTINGS.get('BROKER').get(
+    'MINIMUM_PLAN_AMOUNT', {}).get('HOURLY', 0)
+BROKER_MINIMUM_PLAN_AMOUNT_YEARLY = _SETTINGS.get('BROKER').get(
+    'MINIMUM_PLAN_AMOUNT', {}).get('HOURLY', 0)
 
 #: overrides the implementation of `saas.utils.build_absolute_uri`
 #: This function must return fully qualified URL.
@@ -166,8 +186,10 @@ INACTIVITY_DAYS = _SETTINGS.get('INACTIVITY_DAYS')
 IS_BROKER_CALLABLE = _SETTINGS.get('BROKER').get('IS_INSTANCE_CALLABLE', None)
 MAX_RENEWAL_ATTEMPTS = _SETTINGS.get('MAX_RENEWAL_ATTEMPTS')
 MAX_TYPEAHEAD_CANDIDATES = _SETTINGS.get('MAX_TYPEAHEAD_CANDIDATES')
+MAIL_PROVIDER_DOMAINS = _SETTINGS.get('MAIL_PROVIDER_DOMAINS')
 ORGANIZATION_MODEL = _SETTINGS.get('ORGANIZATION_MODEL')
 PAGE_SIZE = _SETTINGS.get('PAGE_SIZE')
+PHONE_VERIFICATION_BACKEND = _SETTINGS.get('PHONE_VERIFICATION_BACKEND')
 PLATFORM_NAME = _SETTINGS.get('PLATFORM_NAME')
 PROCESSOR = _SETTINGS.get('PROCESSOR')
 PROCESSOR_BACKEND_CALLABLE = _SETTINGS.get('PROCESSOR_BACKEND_CALLABLE')
@@ -207,3 +229,4 @@ TIME_ZONE = getattr(settings, 'TIME_ZONE')
 CONTRIBUTOR = _SETTINGS.get('CONTRIBUTOR')
 MANAGER = _SETTINGS.get('MANAGER')
 PROFILE_URL_KWARG = _SETTINGS.get('PROFILE_URL_KWARG')
+CURRENCY_JSON_PATH = _SETTINGS.get('CURRENCY_JSON_PATH')
