@@ -274,11 +274,15 @@
     Refund.prototype = {
         init: function () {
             var self = this;
-            var submitButton = self.element.find("[type='submit']");
             // Make sure we unbind the previous handler to avoid double submits
-            submitButton.off("click.refund");
-            submitButton.on("click.refund", function() {
-                return self.submit();
+            self.element.off("submit.refund");
+            self.element.on("submit.refund", function() {
+                if( typeof self.element.modal !== 'undefined' ) {
+                    self.element.modal("hide");
+                }
+                self.submit();
+                // prevent the form from submitting with the default action
+                return false;
             });
         },
 
@@ -343,6 +347,7 @@
                     }
                 });
             }
+            return false;
         }
     };
 
@@ -612,7 +617,8 @@
    Redeem.prototype = {
       init: function () {
           var self = this;
-          self.element.find(".submit-code").click(function() {
+          self.element.find(".submit-code").click(function(event) {
+              event.preventDefault();
               var code = self.element.find("[name='code']").val();
               self.redeemCode(code);
               // prevent the form from submitting with the default action
