@@ -1,4 +1,4 @@
-# Copyright (c) 2022, DjaoDjin inc.
+# Copyright (c) 2023, DjaoDjin inc.
 # All rights reserved.
 #
 # Redistribution and use in source and binary forms, with or without
@@ -29,16 +29,21 @@ Urls specific to the hosting site (i.e. broker).
 from ... import settings
 from ...compat import path, re_path
 from ...views.metrics import BalancesView
-from ...views.billing import AllTransactions, ChargeListView, VTChargeView
+from ...views.billing import (AllTransactions, ChargeListView, VTChargeView,
+    ActiveCartItemsView, UserCartItemsView)
 from ...views.download import (BalancesDownloadView, ChargesDownloadView,
     RegisteredDownloadView, TransactionDownloadView)
 
 urlpatterns = [
+    path('billing/cartitems/<slug:user>/',
+         UserCartItemsView.as_view(), name='saas_user_active_carts'),
+    path('billing/cartitems/',
+         ActiveCartItemsView.as_view(), name='saas_active_cartitems'),
     path('billing/charges/download/',
         ChargesDownloadView.as_view(), name='saas_charges_download'),
     path('billing/charges/',
         ChargeListView.as_view(), name='saas_charges'),
-    re_path(r'billing/transactions/((?P<selector>%s)/)?download',
+    re_path(r'billing/transactions/((?P<selector>%s)/)?download/',
         TransactionDownloadView.as_view(),
         name='saas_transactions_download'),
     re_path(r'^billing/transactions/((?P<selector>%s)/)?' %
@@ -47,13 +52,13 @@ urlpatterns = [
     # Organization refers to the subscriber in the following URL pattern.
     re_path(r'^billing/(?P<customer>%s)/vtcharge/' % settings.SLUG_RE,
         VTChargeView.as_view(), name='saas_organization_vtcharge'),
-    re_path(r'^metrics/balances/(?P<report>%s)/((?P<year>\d\d\d\d)/)?download'
+    re_path(r'^metrics/balances/(?P<report>%s)/((?P<year>\d\d\d\d)/)?download/'
         % settings.SLUG_RE,
         BalancesDownloadView.as_view(), name='saas_balances_download'),
     re_path(r'^metrics/balances/(?P<report>%s)/((?P<year>\d\d\d\d)/)?'
         % settings.SLUG_RE,
         BalancesView.as_view(), name='saas_balance'),
-    path('metrics/registered/download',
+    path('metrics/registered/download/',
         RegisteredDownloadView.as_view(),
         name='saas_subscriber_pipeline_download_registered'),
 ]
