@@ -22,7 +22,7 @@
 # OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF
 # ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 
-import datetime, inspect, random, re, sys
+import datetime, inspect, random, re, sys, json
 
 from django.core.exceptions import NON_FIELD_ERRORS
 from django.core.files.storage import default_storage
@@ -35,7 +35,7 @@ from django.utils.timezone import utc, get_current_timezone
 from rest_framework.exceptions import ValidationError
 from rest_framework.settings import api_settings
 from pytz import timezone, UnknownTimeZoneError
-from pytz.tzinfo import DstTzInfo
+from pytz.tzinfo import BaseTzInfo
 
 from .compat import get_model_class, gettext_lazy as _, import_string, six
 
@@ -75,7 +75,7 @@ class SlugTitleMixin(object):
 
 
 def parse_tz(tzone):
-    if issubclass(type(tzone), DstTzInfo):
+    if issubclass(type(tzone), BaseTzInfo):
         return tzone
     if tzone:
         try:
@@ -96,7 +96,9 @@ def as_timestamp(dtime_at=None):
         dtime_at - datetime.datetime(1970, 1, 1, tzinfo=utc)).total_seconds())
 
 
-def datetime_or_now(dtime_at=None):
+def datetime_or_now(dtime_at=None, tzinfo=None):
+    if not tzinfo:
+        tzinfo = utc
     as_datetime = dtime_at
     if isinstance(dtime_at, six.string_types):
         as_datetime = parse_datetime(dtime_at)
@@ -106,9 +108,10 @@ def datetime_or_now(dtime_at=None):
                 as_datetime = datetime.datetime.combine(
                     as_date, datetime.time.min)
     if not as_datetime:
-        as_datetime = datetime.datetime.utcnow().replace(tzinfo=utc)
-    if as_datetime.tzinfo is None:
-        as_datetime = as_datetime.replace(tzinfo=utc)
+        as_datetime = datetime.datetime.now(tz=tzinfo)
+    if (as_datetime.tzinfo is None or
+        as_datetime.tzinfo.utcoffset(as_datetime) is None):
+        as_datetime = as_datetime.replace(tzinfo=tzinfo)
     return as_datetime
 
 
@@ -200,7 +203,7 @@ def get_role_serializer():
     """
     Returns the role serializer model that is active in this project.
     """
-    from . import settings
+    from . import settings #pylint:disable=import-outside-toplevel
     return import_string(settings.ROLE_SERIALIZER)
 
 
@@ -208,7 +211,7 @@ def get_user_serializer():
     """
     Returns the user serializer model that is active in this project.
     """
-    from . import settings
+    from . import settings #pylint:disable=import-outside-toplevel
     return import_string(settings.USER_SERIALIZER)
 
 
@@ -216,7 +219,7 @@ def get_user_detail_serializer():
     """
     Returns the user serializer model that is active in this project.
     """
-    from . import settings
+    from . import settings #pylint:disable=import-outside-toplevel
     return import_string(settings.USER_DETAIL_SERIALIZER)
 
 
@@ -281,7 +284,7 @@ def validate_redirect_url(next_url, sub=False, **kwargs):
             if 'profile' not in kwargs:
                 # XXX deployutils is using :profile but djaoapp still generates
                 #     :organization for the time being.
-                profile = kwargs.get('organization')
+                profile = kwargs['organization']
                 pat_kwargs = kwargs.copy()
                 pat_kwargs.update({'profile': profile})
             path = re.sub(r':(%s)/' % settings.ACCT_REGEX,
@@ -402,3 +405,33 @@ def build_absolute_uri(request, location='/', provider=None, with_scheme=True):
     # If we don't have a `request` object, better to return a URL path
     # than throwing an error.
     return location
+
+
+class CurrencyDataLoader:
+    _currency_data = None
+
+    @classmethod
+    def load_currency_data(cls):
+        if cls._currency_data is None:
+            from . import settings #pylint:disable=import-outside-toplevel
+            with open(settings.CURRENCY_JSON_PATH, 'r') as file:
+                currency_list = json.load(file)
+                cls._currency_data = {currency['cc']:
+                    currency for currency in currency_list}
+        return cls._currency_data
+
+    @classmethod
+    def get_currency_symbols(cls, currency_code):
+        data = cls.load_currency_data()
+        if data:
+            currency = data.get(currency_code.upper())
+            if currency:
+                return currency['symbol']
+        return None
+
+    @classmethod
+    def get_currency_choices(cls):
+        data = cls.load_currency_data()
+        if data:
+            return [(code.lower(), code.lower()) for code in data.keys()]
+        return []
