@@ -59,7 +59,7 @@ from ..forms import (BankForm, CartPeriodsForm, CreditCardForm,
     ImportTransactionForm, RedeemCouponForm, VTChargeForm, WithdrawForm)
 from ..mixins import (BalanceDueMixin, BalanceAndCartMixin, ChargeMixin,
     DateRangeContextMixin, InvoicablesMixin, OrganizationMixin,
-    ProviderMixin, get_charge_context, product_url)
+    ProviderMixin, get_charge_context, product_url, UserMixin)
 from ..models import (CartItem, Charge, Coupon,
     Plan, Price, Subscription, Transaction, UseCharge, get_broker)
 from ..utils import (get_organization_model, update_context_urls,
@@ -537,6 +537,7 @@ reference/djaoapp/latest/api/#listTransfer>`__
         if self.provider.is_broker:
             update_context_urls(context, {
                 'raw_charges': reverse('saas_charges'),
+                'raw_cartitems': reverse('saas_active_cartitems'),
                 'raw_transactions': reverse('saas_broker_transactions')
             })
 
@@ -586,14 +587,13 @@ class CartBaseView(InvoicablesMixin, BalanceAndCartMixin, FormView):
             'submit_title': _("Subscribe")})
         return context
 
-    @staticmethod
-    def get_empty_cart_redirect_url(request, *args, **kwargs):
+    def get_empty_cart_redirect_url(self, request, *args, **kwargs):
         """
         Returns URL to pricing page if the cart is empty.
         """
         #pylint:disable=unused-argument
         if is_authenticated(request):
-            if not CartItem.objects.get_cart(user=request.user).exists():
+            if not self.invoicables:
                 messages.info(request,
                     _("Your Cart is empty. Please add some items to your cart"
 " before you check out."))
@@ -1144,3 +1144,36 @@ djaodjin-saas/tree/master/saas/templates/saas/billing/import.html>`__).
     def get_success_url(self):
         return reverse('saas_transfer_info',
             kwargs=self.get_url_kwargs(**self.kwargs))
+
+
+class ActiveCartItemsView(ProviderMixin, TemplateView):
+    """
+    Display users with active carts. - i.e. they haven't been checked out yet.
+    """
+    template_name = 'saas/billing/cartitems/index.html'
+
+    def get_context_data(self, **kwargs):
+        context = super(ActiveCartItemsView, self).get_context_data(**kwargs)
+        update_context_urls(context, {
+            'saas_api_cartitems': reverse('saas_api_cartitems'),
+            'saas_api_pricing': reverse('saas_api_pricing'),
+        })
+        return context
+
+
+class UserCartItemsView(ProviderMixin, UserMixin, TemplateView):
+    """
+    Display a users active cart. - i.e. checkout in progress.
+    """
+    template_name = 'saas/billing/cartitems/user.html'
+
+    def get_context_data(self, **kwargs):
+        context = super(UserCartItemsView, self).get_context_data(**kwargs)
+        user = self.user
+        update_context_urls(context, {
+            'saas_api_user_cartitems': reverse('saas_api_user_cartitems',
+                args=(user,)),
+            'saas_api_pricing': reverse('saas_api_pricing'),
+            'saas_api_cartitems': reverse('saas_api_cartitems'),
+        })
+        return context
