@@ -1,4 +1,4 @@
-# Copyright (c) 2022, DjaoDjin inc.
+# Copyright (c) 2023, DjaoDjin inc.
 # All rights reserved.
 #
 # Redistribution and use in source and binary forms, with or without
@@ -32,7 +32,7 @@ from .download import CSVDownloadView
 from .. import settings
 from ..api.metrics import LifetimeValueMetricMixin
 from ..compat import reverse
-from ..mixins import CouponMixin, ProviderMixin, MetricsMixin
+from ..mixins import CouponMixin, DateRangeContextMixin, ProviderMixin
 from ..models import CartItem, Plan
 from ..utils import datetime_or_now, update_context_urls
 
@@ -42,7 +42,8 @@ class SubscribersActivityView(ProviderMixin, TemplateView):
     template_name = 'saas/metrics/activity.html'
 
     def get_context_data(self, **kwargs):
-        context = super().get_context_data(**kwargs)
+        context = super(
+            SubscribersActivityView, self).get_context_data(**kwargs)
         ends_at = datetime_or_now()
         expires_at = ends_at - relativedelta(years=1)
         context.update({
@@ -226,7 +227,7 @@ reference/djaoapp/latest/api/#listPlanMetric>`__
         return context
 
 
-class RevenueMetricsView(MetricsMixin, TemplateView):
+class RevenueMetricsView(DateRangeContextMixin, ProviderMixin, TemplateView):
     """
     Reports cash flow and revenue in currency units.
 
@@ -270,22 +271,56 @@ reference/djaoapp/latest/api/#listCustomerMetric>`__
                 "title": "Amounts",
                 "unit": unit,
                 "location": reverse('saas_api_revenue',
+                    args=(self.organization,)),
+                "download": reverse('saas_metrics_revenue_download',
                     args=(self.organization,))
             }, {
                 "key": "balances",
                 "title": "Balances",
                 "unit": unit,
                 "location": reverse('saas_api_balances',
+                    args=(self.organization,)),
+                "download": reverse('saas_metrics_balances_download',
                     args=(self.organization,))
             }, {
                 "key": "customer",
                 "title": "Customers",
                 "location": reverse('saas_api_customer',
+                    args=(self.organization,)),
+                "download": reverse('saas_metrics_customers_download',
                     args=(self.organization,))
             }, {
                 "key": "plan",
                 "title": "Active subscribers",
                 "location": reverse('saas_api_metrics_plans',
+                    args=(self.organization,)),
+                "download": reverse('saas_metrics_plans_download',
                     args=(self.organization,))
             }])})
         return context
+
+class BalancesDueView(ProviderMixin, TemplateView):
+    """
+    Customers with balances due
+
+    Template:
+
+    To edit the layout of this page, create a local \
+    ``saas/metrics/balances_due.html`` (`example <https://github.com/djaodjin\
+/djaodjin-saas/tree/master/saas/templates/saas/metrics/balances_due.html>`__).
+
+    Template context:
+      - ``organization`` The provider object
+      - ``request`` The HTTP request object
+    """
+    template_name = 'saas/metrics/balances_due.html'
+
+    def get_context_data(self, **kwargs):
+        context = super(BalancesDueView, self).get_context_data(
+            **kwargs)
+        update_context_urls(context, {
+            'provider': {
+                'api_metrics_balances_due': reverse(
+                    'saas_api_metrics_balances_due', args=(self.provider,))}
+        })
+        return context
